#!/bin/sh
# Next line a tcl comment \
exec wish -f "$0" ${1+"$@"}

# Triple-helical collagen building script by Jan K. Rainey.
# Copyright 2003-2013 Jan K. Rainey.
#
# If you use this program, please reference:
# J.K. Rainey and M.C. Goh. (2004) An interactive triple-helical
# collagen builder. Bioinformatics. 20: 2458-2459.
# Also crucial to any results you may obtain is:
# J.K. Rainey and M.C. Goh. (2002) A statistically derived
# parameterization for the collagen triple-helix. Protein
# Science. 11: 2748-2754.
# Side-chain prediction and terminal atom statistics rely
# upon: J.K. Rainey and M.C. Goh. (2004) Statistically based
# reduced representation of amino acid side chains.Journal 
# of Chemical Information & Computer Sciences. 44: 817-830.
#
# Version: 1.07
# Script built for Tcl/Tk 8.5.4
# Download Tcl/Tk for free at http://www.tcl.tk
# See the accompanying PDF file for documentation.
#
# Contact J.K.R. at:
# jan.rainey@dal.ca
# Please report any bugs or inform me if you make major modifications.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


# The following script is not yet (fully) documented. Please be patient
# and check for updates!

proc bgerror {message} {
  
	global errorInfo
	
	set errtext "Error encountered - usually due to trying to build a second triple-helical molecule without restarting. If this is not the case, please e-mail jan.rainey@dal.ca to report a bug. \n\n Do you want to restart & try again?"
  set outfile [open {error_THeBuScr.out} w]
	puts $outfile "$message"
	puts $outfile "$errorInfo"
	close $outfile
	set restartbut [tk_messageBox -default yes -type yesno -icon question -message "$errtext" -parent . ]
	if {$restartbut == {yes}} {
		destroy .
		exec {THeBuScr.1.07.tcl}
	}
}

proc setupchvar {} {
  global chvar
	set chvar(A_NT) -1
	set chvar(A_CT) -1
	set chvar(B_NT) -1
	set chvar(B_CT) -1
	set chvar(C_NT) -1
	set chvar(C_CT) -1
		
}

proc openseq {filename} {

  set fileid [open $filename r]
	set theseq {}
	while {[gets $fileid currline] >=0} {
	  if {[string compare -length 1 {>} $currline] != 0} {
		  set theseq "$theseq$currline"
		}
	} 
	close $fileid
	set lowflag 0
  if {[string is upper $theseq] == 0} {
	  set lowflag 1
	}
	set toret $theseq
	if {$lowflag == 1} {
		unset -nocomplain converttoupper
   	set converttoupper [tk_messageBox -default yes -type yesno -icon question -message "This sequence contains lower case amino acid letters. Convert to upper?" -parent . ]
		if {$converttoupper == {yes}} {
	  	set toret [string toupper $theseq]
		}
	} 
	return "$toret"
}


proc findgxx {currchain startth endth} {
  upvar $endth ev
	upvar $startth sv
	
	set gchain {}
	set strlen "[expr [string length $currchain] - 1]"
	for {set i 0} {$i<$strlen} {incr i} {
		if {[string index $currchain $i] == "G"} then {
			set currchar {G}
		} else {
		  set currchar {_}
		}
		set gchain "$gchain$currchar"
	}
  
	set ev [expr [string last G__G__G__G $gchain] + 12]
	set sv [expr [string last ___ [string range $gchain 0 [expr "$ev" - 1]] ] + 4]
	if {[string index $currchain 0] == "G" & $sv == 3} {
	  set sv 1
	}
	set sv [expr [string first G__ [string range $gchain [expr "$sv" - 1] [expr "$ev" - 1]] ] + $sv]

}

proc hltselec { twin } {
  
	global ntvar ctvar
	
	$twin tag delete redlett
	$twin tag configure redlett -foreground red3
  $twin tag add redlett [expr (round(floor(($ntvar - 1) / 100)) + 3)].[expr ((($ntvar-1) % 100) + 5)] [expr (round(floor(($ctvar - 1) / 100)) + 3)].[expr ((($ctvar-1) % 100) + 6)]
	
	$twin tag raise redlett 
	$twin tag raise blacklett

}


proc goterm {nn nc topwin} {
  global chvar ntvar ctvar
	set chvar($nn) $ntvar
	set chvar($nc) $ctvar
  destroy "$topwin"
}	

proc dispseq {tw currchain startch endch namen namec} {
  
	global tcl_platform  
	global ntvar ctvar
  set	ntvar $startch
	set ctvar $endch

	set strlen "[expr [string length $currchain] - 1]"
  set numlines [expr {int($strlen / 100)} + 1]
  
	toplevel $tw
  wm title $tw "N- and C-terminal selection"
	focus $tw
	grab $tw
		

  #MS Windows appears to have a very different font metric.
	#Hopefully this fix works in all cases...if it's too small on your
	#screen, feel free to up the fontsize.
	set fontsize 12
	if {$tcl_platform(platform) == {windows}} {
	  if {[winfo screenwidth $tw] <= 800} {
		  set fontsize 8
		} else {
		  set fontsize 10
		}
	}
  text $tw.textwin -relief raised -bd 2 -bg white -font "-family courier -size $fontsize" -width 105 -height [expr $numlines + 8]
  pack $tw.textwin -side left 
	$tw.textwin tag configure bluelett -foreground blue
	$tw.textwin tag configure blacklett -foreground black
	
	$tw.textwin insert end [format "%15d" 10]
  for {set i 2} {$i<=10} {incr i} {
	  $tw.textwin insert end [format "%10d" [expr $i * 10]]
	}
	$tw.textwin insert end "\n[format "%15s" |]"
  for {set i 2} {$i<=10} {incr i} {
	  $tw.textwin insert end [format "%10s" |]
	}
	$tw.textwin insert end \n
	$tw.textwin tag add blacklett 1.0 2.end

	for {set i 0} {$i<$numlines} {incr i} {
	  $tw.textwin insert end [format "%-5d" [expr $i * 100 + 1]]
		$tw.textwin tag add blacklett [expr $i + 3].0 [expr $i + 3].4
		$tw.textwin insert end "[string range $currchain [expr $i * 100] [expr ($i+1)*100 - 1]]\n"
		$tw.textwin tag add bluelett [expr $i + 3].5 [expr $i + 3].end
  }

  hltselec $tw.textwin
	label $tw.textwin.ntlab -text "N-terminal: "
  label $tw.textwin.ctlab -text "C-terminal: "
	entry $tw.textwin.ntentry -width 8 -relief sunken -textvariable ntvar 
	entry $tw.textwin.ctentry -width 8 -relief sunken -textvariable ctvar
	button $tw.textwin.updatebut -text {Update sequence highlighting} -command "hltselec $tw.textwin"
	$tw.textwin insert end "\n"
	$tw.textwin window create end -window $tw.textwin.ntlab
  $tw.textwin window create end -window $tw.textwin.ntentry
	$tw.textwin window create end -window $tw.textwin.ctlab
  $tw.textwin window create end -window $tw.textwin.ctentry
	$tw.textwin window create end -window $tw.textwin.updatebut
	$tw.textwin insert end "\n"
	$tw.textwin configure -state disabled
	button $tw.textwin.ok -text "Use these termini" -command "goterm $namen $namec $tw"
	button $tw.textwin.cancel -text Cancel -command "destroy $tw"
	$tw.textwin window create end -window $tw.textwin.ok 
	$tw.textwin window create end -window $tw.textwin.cancel
  tkwait window $tw

}

proc orderchains { } {

  global chaina chainb chainc chvar
  global chaina_raw chainb_raw chainc_raw
	global cha_file chb_file chc_file

	
	set numta [expr ([string length $chaina] / 3)]
  set numtb [expr ([string length $chainb] / 3)]
	set numtc [expr ([string length $chainc] / 3)]
	if {$numtb > $numta || $numtc > $numtb || $numtc > $numta} {
		set orderch [tk_messageBox -type yesno -icon question -message "Chains have following numbers of triplets:\n A: $numta\n B: $numtb\n C: $numtc\n\n Reorder so chain A is longest?" -parent . ]
		
		if {$orderch == {yes}} {	  

			if {$numtb > $numta} {
				set chtemp $chaina
				set chaina $chainb
				set chainb $chtemp
				set chtemp $chaina_raw
				set chaina_raw $chainb_raw
				set chainb_raw $chtemp
				set ftemp $cha_file
				set cha_file $chb_file
				set chb_file $ftemp
				set numtemp $numta
				set numta $numtb
				set numtb $numtemp
				set nttemp $chvar(A_NT)
				set cttemp $chvar(A_CT)
				set chvar(A_NT) $chvar(B_NT)
				set chvar(A_CT) $chvar(B_CT)
				set chvar(B_NT) $nttemp
				set chvar(B_CT) $cttemp
				
			}
			if {$numtc > $numta} {
				set chtemp $chaina
				set chaina $chainc
				set chainc $chtemp
				set chtemp $chaina_raw
				set chaina_raw $chainc_raw
				set chainc_raw $chtemp
				set ftemp $cha_file
				set cha_file $chc_file
				set chc_file $ftemp
				set numtemp $numta
				set numta $numtc
				set numtc $numtemp
				set nttemp $chvar(A_NT)
				set cttemp $chvar(A_CT)
				set chvar(A_NT) $chvar(C_NT)
				set chvar(A_CT) $chvar(C_CT)
				set chvar(C_NT) $nttemp
				set chvar(C_CT) $cttemp

			} 
  		if {$numtc > $numtb} {
				set chtemp $chainc
				set chainc $chainb
				set chainb $chtemp
				set chtemp $chainc_raw
				set chainc_raw $chainb_raw
				set chainb_raw $chtemp
				set ftemp $chc_file
				set chc_file $chb_file
				set chb_file $ftemp
				set numtemp $numtb
				set numtb $numtc
				set numtc $numtemp
				set nttemp $chvar(C_NT)
				set cttemp $chvar(C_CT)
				set chvar(C_NT) $chvar(B_NT)
				set chvar(C_CT) $chvar(B_CT)
				set chvar(B_NT) $nttemp
				set chvar(B_CT) $cttemp

			}
			set numta [expr ([string length $chaina] / 3)]
  		set numtb [expr ([string length $chainb] / 3)]
			set numtc [expr ([string length $chainc] / 3)]
    }
	}
}

proc readpropensities { } {
  global propdata maxprop minprop
	
	set maxprop 0
	set minprop 100
	
	set fileid [open {THpropensities.dat} r]
	while {[gets $fileid currline] >=0} {
	  if {[string compare -length 1 {;} $currline] != 0 && [string length  $currline] > 3} {
		  scan $currline "%s %f" currtrip currprop
			if {[string length $currtrip] == 3} {
			  set propdata($currtrip) $currprop
				if {$currprop > $maxprop} {
				  set maxprop $currprop
				}
				if {$currprop < $minprop} {
				  set minprop $currprop
				}
			}
			 
		}
	} 
	close $fileid
}

proc getprop { currtrip } {
  global propdata
	
	if {[string index $currtrip 0] != "G"} {
	  puts stdout "Non-Gly in G position - returning 0 propensity. Triplet: $currtrip"
	  return 0
	}
	
	if {[string match ??P $currtrip] == 1} {
	  set currtrip "[string range $currtrip 0 1]O"
	} 
	
	set currsearch [array get propdata $currtrip]
	
	if {[llength $currsearch] > 1} {
	  return [lindex $currsearch 1]
	} else {
    set tripx "G[string index $currtrip 1]O"
		set tripy "GP[string index $currtrip 2]"
		set searchx [array get propdata $tripx]
		set searchy [array get propdata $tripy]
		set tmgpo [array get propdata "GPO"]
		if {[llength $searchx] > 1 && [llength $searchy] > 1} {
		  return [expr ([lindex $searchx 1] + [lindex $searchy 1] - [lindex $tmgpo 1])]
		} else {
			puts stdout "Non-standard 1 letter code? - returning 0 propensity. Triplet: $currtrip"
			return 0	
		}
	}
  puts stdout "Non-standard triplet - unable to determine propensity. Returning 0. Triplet: $currtrip"
	return 0	
}

proc trippropensities { cha chb chc } {
  
	global tripa_tms tripb_tms tripc_tms
	global mean_tm
	
	set numta [expr ([string length $cha] / 3)]
  set numtb [expr ([string length $chb] / 3)]
	set numtc [expr ([string length $chc] / 3)]
	set mean_tm 0
	
	for {set i 0} {$i < $numta} {incr i} {
	  set currtrip [string range $cha [expr ("$i" * 3)] [expr ("$i" * 3) + 2]]
		set currprop [getprop $currtrip]
		lappend tripa_tms $currprop 
		set mean_tm [expr $mean_tm + $currprop]
	}
	for {set i 0} {$i < $numtb} {incr i} {
	  set currtrip [string range $chb [expr ("$i" * 3)] [expr ("$i" * 3) + 2]]
		set currprop [getprop $currtrip]
		lappend tripb_tms $currprop 
		set mean_tm [expr $mean_tm + $currprop]
	}
	for {set i 0} {$i < $numtc} {incr i} {
	  set currtrip [string range $chc [expr ("$i" * 3)] [expr ("$i" * 3) + 2]]
 		set currprop [getprop $currtrip]
		lappend tripc_tms $currprop 
		set mean_tm [expr $mean_tm + $currprop]
	}
	set mean_tm [expr $mean_tm / ($numta + $numtb + $numtc)]
 
}

proc retGXY { aanum } {
  set currnum [expr $aanum % 3]
	if { $currnum == 0 } {
	  return G
	} elseif { $currnum == 1 } {
	  return X
	}
	return Y
}

proc boxcarpropensities { cha chb chc } {

  global tripa_tms tripb_tms tripc_tms

  # Starting at triplet 1:
	# G1 = 0, X1 = 1, Y1 = 2, G2 = 3, X2 = 4, etc.
	# This corresponds exactly with cha chb chc residue codes at string
	# position 0, 1, 2, 3, etc.
	
	global cha_tms chb_tms chc_tms
  global cha_codes chb_codes chc_codes
		
	set numta [expr ([string length $cha] / 3)]
  set numtb [expr ([string length $chb] / 3)]
	set numtc [expr ([string length $chc] / 3)]
  
  if {$numtb < $numtc} {
	  puts stdout "Abnormal helix (length chain B < chain C). Only calculating propensities up to length of chain B."
		set numtc $numtb
	}
	if {$numtc < $numtb} {
	  puts stdout "Abnormal helix (length chain C < chain B). Only calculating propensities up to length of chain C."
		set numtb $numtc
	}
	#set G1 and X1 of chain A to 0. G1 of chain B to 0.
	set cha_tms(0) 0
	set cha_codes(0) G
	set cha_tms(1) 0
	set cha_codes(1) X
	set chb_tms(0) 0
	set chb_codes(0) G
		
	# Do the finicky stuff at the N-terminal as described in the manual
	
	#First, set Y0(A), X0(B), G0(C)
	# after these set statements, $a_num, $b_num & $c_num increment automatically
	set a_num 2 
	set b_num 1
	set c_num 0
	set Ymults [list [expr 24.0/9] [expr 21.0/9] [expr 12.0/9] [expr 3.0/9]]
	set Xmults [list [expr 25.0/9] [expr 18.0/9] [expr 9.0/9] [expr 1.0/9]]
	set Gmults [list [expr 24.0/9] [expr 15.0/9] [expr 6.0/9] 0.0] 
	
	set CurrTm 0.0
	set CurrCount 0.0
	for {set i 0} {$i < 4} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms $i] * [lindex $Ymults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms $i] * [lindex $Xmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms $i] * [lindex $Gmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]

	#Now set G1(A), Y0(B), X0(C)
	incr a_num
	incr b_num
	incr c_num

	set Gmults [linsert $Gmults 0 [expr 21.0/9]] 

	set CurrTm 0
	set CurrCount 0
	for {set i 0} {$i < 4} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms $i] * [lindex $Gmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms $i] * [lindex $Ymults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms $i] * [lindex $Xmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]
	
	#Now set X1(A), G1(B), Y0(C)
	incr a_num
	incr b_num
	incr c_num

	set Xmults [linsert $Xmults 0 [expr 18.0/9]]
	lappend Ymults 0.0

	set CurrTm 0
	set CurrCount 0
	for {set i 0} {$i < 5} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms $i] * [lindex $Xmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms $i] * [lindex $Gmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms $i] * [lindex $Ymults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]
	
	#Now set Y1(A), X1(B), G1(C)
	incr a_num
	incr b_num
	incr c_num

  set Ymults [linsert $Ymults 0 [expr 15.0/9]] 

	set CurrTm 0
	set CurrCount 0
	for {set i 0} {$i < 5} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms $i] * [lindex $Ymults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms $i] * [lindex $Xmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms $i] * [lindex $Gmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]
	
	#Now set G2(A), Y1(B), X1(C)
	incr a_num
	incr b_num
	incr c_num

	set Gmults [linsert $Gmults 0 [expr 12.0/9]]

	set CurrTm 0
	set CurrCount 0
	for {set i 0} {$i < 5} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms $i] * [lindex $Gmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms $i] * [lindex $Ymults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms $i] * [lindex $Xmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]
	
	#Now set X2(A), G2(B), Y1(C)
	incr a_num
	incr b_num
	incr c_num

	set Xmults [linsert $Xmults 0 [expr 9.0/9]]

	set CurrTm 0
	set CurrCount 0
	for {set i 0} {$i < 6} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms $i] * [lindex $Xmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms $i] * [lindex $Gmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms $i] * [lindex $Ymults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]

	#Now set Y2(A), X2(B), G2(C)
	incr a_num
	incr b_num
	incr c_num

	set Ymults [linsert $Ymults 0 [expr 6.0/9]]

	set CurrTm 0
	set CurrCount 0
	for {set i 0} {$i < 6} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms $i] * [lindex $Ymults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms $i] * [lindex $Xmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms $i] * [lindex $Gmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]
	
	#Now set G3(A), Y2(B), X2(C)
	incr a_num
	incr b_num
	incr c_num

	set Gmults [linsert $Gmults 0 [expr 3.0/9]]

	set CurrTm 0
	set CurrCount 0
	for {set i 0} {$i < 6} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms $i] * [lindex $Gmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms $i] * [lindex $Ymults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms $i] * [lindex $Xmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]
  
  # Now assigned all the way up to:
	# Chain A - Gly0 & X0 = 0, Y0 -> Gly3 = Tm*
	#	Chain B - Gly0 = 0, X0 -> Y2 = Tm*
	#	Chain C - Gly0 -> X2 = Tm*
	
	# Next, assign all Tm* possible by standard sliding window boxcar weighting
	
	if {$numta > $numtb} {
	  set endtrip [expr $numtb - 2]
	} else {
	  set endtrip [expr $numtb - 3]
	}
	
	for {set Trip 1} {$Trip < $endtrip} {incr Trip} {
	  set startelm [expr $Trip - 1]
		set endelm [expr $Trip + 3]
		set Cha_vals [lrange $tripa_tms $startelm $endelm]
		set Chb_vals [lrange $tripb_tms $startelm $endelm]
		set Chc_vals [lrange $tripc_tms $startelm $endelm]
	  for {set WindPos 0} {$WindPos < 3} {incr WindPos} {
		  if {$WindPos == 0} {
			  set Cha_coef [list 1 3 3 2 0]
				set Chb_coef [list 2 3 3 1 0]
				set Chc_coef [list 3 3 3 0 0]
			} elseif {$WindPos == 1} {
			  set Cha_coef [list 0 3 3 3 0]
				set Chb_coef [list 1 3 3 2 0]
				set Chc_coef [list 2 3 3 1 0]
			} else {
			  set Cha_coef [list 0 2 3 3 1]
				set Chb_coef [list 0 3 3 3 0]
				set Chc_coef [list 1 3 3 2 0]
			}
			set Cha_cont 0
			set Chb_cont 0
			set Chc_cont 0
			set TotNumCont 0
			for {set Pos 0} {$Pos < 5} {incr Pos} {
				set Cha_cont [expr ($Cha_cont + ([lindex $Cha_coef $Pos] * [lindex $Cha_vals $Pos]))]
	  		set TotNumCont [expr ($TotNumCont + ([lindex $Cha_coef $Pos] * ([lindex $Cha_vals $Pos] != 0)))]

				#This if required to handle unequal chain lengths at C-terminal end...
				if {$Pos < 4} {
  				set Chb_cont [expr ($Chb_cont + ([lindex $Chb_coef $Pos] * [lindex $Chb_vals $Pos]))]
	  			set Chc_cont [expr ($Chc_cont + ([lindex $Chc_coef $Pos] * [lindex $Chc_vals $Pos]))]
  				set TotNumCont [expr ($TotNumCont + ([lindex $Chb_coef $Pos] * ([lindex $Chb_vals $Pos] != 0)))]
	  	  	set TotNumCont [expr ($TotNumCont + ([lindex $Chc_coef $Pos] * ([lindex $Chc_vals $Pos] != 0)))]
				}	
  		}
      set Tmcurr($WindPos) [expr ($Cha_cont + $Chb_cont + $Chc_cont) / $TotNumCont]      
		}
		lappend Tms0 $Tmcurr(0)
		lappend Tms1 $Tmcurr(1)
		lappend Tms2 $Tmcurr(2)
	}
  
	#add last two window positions manually if chain A = chain B
  if {$numtb == $numta} {
 	  set startelm [expr $numtb - 4]
		set Cha_vals [lrange $tripa_tms $startelm end]
		set Chb_vals [lrange $tripb_tms $startelm end]
		set Chc_vals [lrange $tripc_tms $startelm end]
	  for {set WindPos 0} {$WindPos < 2} {incr WindPos} {
		  if {$WindPos == 0} {
			  set Cha_coef [list 1 3 3 2]
				set Chb_coef [list 2 3 3 1]
				set Chc_coef [list 3 3 3 0]
			} else {
			  set Cha_coef [list 0 3 3 3]
				set Chb_coef [list 1 3 3 2]
				set Chc_coef [list 2 3 3 1]
			} 
			set Cha_cont 0
			set Chb_cont 0
			set Chc_cont 0
			set TotNumCont 0
			for {set Pos 0} {$Pos < 4} {incr Pos} {
				set Cha_cont [expr ($Cha_cont + ([lindex $Cha_coef $Pos] * [lindex $Cha_vals $Pos]))]
				set Chb_cont [expr ($Chb_cont + ([lindex $Chb_coef $Pos] * [lindex $Chb_vals $Pos]))]
				set Chc_cont [expr ($Chc_cont + ([lindex $Chc_coef $Pos] * [lindex $Chc_vals $Pos]))]
				set TotNumCont [expr ($TotNumCont + ([lindex $Cha_coef $Pos] * ([lindex $Cha_vals $Pos] != 0)))]
  			set TotNumCont [expr ($TotNumCont + ([lindex $Chb_coef $Pos] * ([lindex $Chb_vals $Pos] != 0)))]
				set TotNumCont [expr ($TotNumCont + ([lindex $Chc_coef $Pos] * ([lindex $Chc_vals $Pos] != 0)))]
			}
      set Tmcurr($WindPos) [expr ($Cha_cont + $Chb_cont + $Chc_cont) / $TotNumCont]      
		}
		lappend Tms0 $Tmcurr(0)
		lappend Tms1 $Tmcurr(1)
	}
	  	
	for {set Trip 3} {$Trip < [expr $numtb - 3]} {incr Trip} {
	  
		for {set WindPos 0} {$WindPos < 3} {incr WindPos} {
		
		  if {$WindPos == 0} {
				
				set CurrTm [expr [lindex $Tms0 [expr $Trip - 3]] + [lindex $Tms1 [expr $Trip - 3]] + [lindex $Tms2 [expr $Trip - 3]]]
				set CurrTm [expr $CurrTm + [lindex $Tms0 [expr $Trip - 2]] + [lindex $Tms1 [expr $Trip - 2]] + [lindex $Tms2 [expr $Trip - 2]]]
				set CurrTm [expr $CurrTm + [lindex $Tms0 [expr $Trip - 1]] + [lindex $Tms1 [expr $Trip - 1]] + [lindex $Tms2 [expr $Trip - 1]]]
			
			} elseif {$WindPos == 1} {					
				
				set CurrTm [expr [lindex $Tms1 [expr $Trip - 3]] + [lindex $Tms2 [expr $Trip - 3]]]
				set CurrTm [expr $CurrTm + [lindex $Tms0 [expr $Trip - 2]] + [lindex $Tms1 [expr $Trip - 2]] + [lindex $Tms2 [expr $Trip - 2]]]
				set CurrTm [expr $CurrTm + [lindex $Tms0 [expr $Trip - 1]] + [lindex $Tms1 [expr $Trip - 1]] + [lindex $Tms2 [expr $Trip - 1]]]
				set CurrTm [expr $CurrTm + [lindex $Tms0 $Trip]]
    	
      } else {
				
				set CurrTm [lindex $Tms2 [expr $Trip - 3]]
				set CurrTm [expr $CurrTm + [lindex $Tms0 [expr $Trip - 2]] + [lindex $Tms1 [expr $Trip - 2]] + [lindex $Tms2 [expr $Trip - 2]]]
				set CurrTm [expr $CurrTm + [lindex $Tms0 [expr $Trip - 1]] + [lindex $Tms1 [expr $Trip - 1]] + [lindex $Tms2 [expr $Trip - 1]]]
				set CurrTm [expr $CurrTm + [lindex $Tms0 $Trip] + [lindex $Tms1 $Trip]]
      
			}
	  	
			set CurrTm [expr $CurrTm / 9]
    	
			incr a_num
			incr b_num
			incr c_num
			
			set cha_tms($a_num) $CurrTm
			set cha_codes($a_num) [retGXY "$a_num"]
			set chb_tms($b_num) $CurrTm
			set chb_codes($b_num) [retGXY "$b_num"]
			set chc_tms($c_num) $CurrTm
			set chc_codes($c_num) [retGXY "$c_num"]
			
		}
	}
  # Do final boxcar if A is longer than B.
  if {$numta > $numtb} {
  	set Trip [expr $numtb - 3]  
		set CurrTm [expr [lindex $Tms0 [expr $Trip - 3]] + [lindex $Tms1 [expr $Trip - 3]] + [lindex $Tms2 [expr $Trip - 3]]]
		set CurrTm [expr $CurrTm + [lindex $Tms0 [expr $Trip - 2]] + [lindex $Tms1 [expr $Trip - 2]] + [lindex $Tms2 [expr $Trip - 2]]]
		set CurrTm [expr $CurrTm + [lindex $Tms0 [expr $Trip - 1]] + [lindex $Tms1 [expr $Trip - 1]] + [lindex $Tms2 [expr $Trip - 1]]]
		set CurrTm [expr $CurrTm / 9]
		
		incr a_num
		incr b_num
		incr c_num

		set cha_tms($a_num) $CurrTm
		set cha_codes($a_num) [retGXY "$a_num"]
		set chb_tms($b_num) $CurrTm
		set chb_codes($b_num) [retGXY "$b_num"]
		set chc_tms($c_num) $CurrTm
		set chc_codes($c_num) [retGXY "$c_num"]

	}
		
	# Now, work backwards from end of chain to fill in remaining Tms.
	# Add (chain A = chain B) > chain C?
	
	set fina_num [expr $a_num + 1]
	
	if {$numta > $numtb} {
	
	  # Ch A - X and Y at trip = numta are 0
 		# Ch C - Y at trip = numtc are 0
    set cha_tms([expr (($numta - 1) * 3 + 1)]) 0
	  set cha_codes([expr (($numta - 1) * 3 + 1)]) X
		set cha_tms([expr (($numta - 1) * 3 + 2)]) 0
		set cha_codes([expr (($numta - 1) * 3 + 2)]) Y
		set chc_tms([expr (($numtc - 1) * 3 + 2)]) 0
		set chc_codes([expr (($numtc - 1) * 3 + 2)]) Y
				
		set a_num [expr ($numta - 1) * 3]
		set b_num [expr ($numtb - 1) * 3 + 2]
		set c_num [expr ($numtc - 1) * 3 + 1] 
			
	} elseif {$numta == $numtb} {
	  
	  # Ch B - Y at trip = numtb are 0
 		# Ch C - X and Y at trip = numtc are 0
    set chb_tms([expr (($numtb - 1) * 3 + 2)]) 0
	  set chb_codes([expr (($numtb - 1) * 3 + 2)]) Y
		set chc_tms([expr (($numtc - 1) * 3 + 1)]) 0
		set chc_codes([expr (($numtc - 1) * 3 + 1)]) X
		set chc_tms([expr (($numtc - 1) * 3 + 2)]) 0
		set chc_codes([expr (($numtc - 1) * 3 + 2)]) Y
		
		set a_num [expr ($numta - 1) * 3 + 2]
		set b_num [expr ($numtb - 1) * 3 + 1]
		set c_num [expr ($numtc - 1) * 3] 
		
  		 
	} else {
	  
		puts stdout "Abnormal helix (length Chain A < Chain B????). Did not set Tm* for C-terminal region."
		return
  
	} 	

  #For following comments N = numtb, if numta > numtb then numta = N+1

  #0th list item corresponds to N+1 - only used if chain a > chains b/c

	set Gmults [list [expr 24.0/9] [expr 21.0/9] [expr 12.0/9] [expr 3.0/9] 0.0]
  set Ymults [list [expr 21.0/9] [expr 24.0/9] [expr 15.0/9] [expr 6.0/9] 0.0]
	set Xmults [list [expr 18.0/9] [expr 25.0/9] [expr 18.0/9] [expr 9.0/9] [expr 1.0/9]] 
	
  #First set Ch A G(N+1), B Y(N), C X(N) if appropriate:
	if {$numta > $numtb} {

		set CurrTm [expr [lindex $tripa_tms [expr $numta - 1]] * [lindex $Gmults 0]]
		set CurrCount [lindex $Gmults 0]
		for {set i 1} {$i < 5} {incr i} {
	  	set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
    	set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
			set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
    	set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
			set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
			set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
		}

		set CurrTm [expr $CurrTm / $CurrCount]
		set cha_tms($a_num) $CurrTm
		set cha_codes($a_num) [retGXY "$a_num"]
		set chb_tms($b_num) $CurrTm
		set chb_codes($b_num) [retGXY "$b_num"]
		set chc_tms($c_num) $CurrTm
		set chc_codes($c_num) [retGXY "$c_num"]
		incr a_num -1
		incr b_num -1
    incr c_num -1
	}
	
  #Now set Ch A Y(N), B X(N), C G(N)
	set Gmults [linsert $Gmults 0 [expr 15.0/9]]

	set CurrTm 0
	set CurrCount 0
	if {$numta > $numtb} {
		set CurrTm [expr [lindex $tripa_tms [expr $numta - 1]] * [lindex $Ymults 0]]
		set CurrCount [lindex $Ymults 0]
  }
	
	for {set i 1} {$i < 5} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]
	
	#Now set Ch A X(N), B G(N), C Y(N-1)
	incr a_num -1
	incr b_num -1
	incr c_num -1
	
	set Ymults [linsert $Ymults 0 [expr 12.0/9]]

	set CurrTm 0
	set CurrCount 0
	if {$numta > $numtb} {
		set CurrTm [expr [lindex $tripa_tms [expr $numta - 1]] * [lindex $Xmults 0]]
		set CurrCount [lindex $Xmults 0]
  }
	
	for {set i 1} {$i < 5} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]
	
	#Now set Ch A G(N), B Y(N-1), C X(N-1)
	incr a_num -1
	incr b_num -1
	incr c_num -1
	
	set Xmults [linsert $Xmults 0 [expr 9.0/9]]

	set CurrTm 0
	set CurrCount 0
	if {$numta > $numtb} {
		set CurrTm [expr [lindex $tripa_tms [expr $numta - 1]] * [lindex $Gmults 0]]
		set CurrCount [lindex $Gmults 0]
  }
	
	for {set i 1} {$i < 6} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]

	#Now set Ch A Y(N-1), B X(N-1), C G(N-1)
	incr a_num -1
	incr b_num -1
	incr c_num -1
	
	set Gmults [linsert $Gmults 0 [expr 6.0/9]]

	set CurrTm 0
	set CurrCount 0
	if {$numta > $numtb} {
		set CurrTm [expr [lindex $tripa_tms [expr $numta - 1]] * [lindex $Ymults 0]]
		set CurrCount [lindex $Ymults 0]
  }
	
	for {set i 1} {$i < 6} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]		

  #Now set Ch A X(N-1), B G(N-1), C Y(N-2)
	incr a_num -1
	incr b_num -1
	incr c_num -1
	
	set Ymults [linsert $Ymults 0 [expr 3.0/9]]

	set CurrTm 0
	set CurrCount 0
	if {$numta > $numtb} {
		set CurrTm [expr [lindex $tripa_tms [expr $numta - 1]] * [lindex $Xmults 0]]
		set CurrCount [lindex $Xmults 0]
  }
	
	for {set i 1} {$i < 6} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]		

	#Now set Ch A G(N-1), B Y(N-2), C X(N-2)
	incr a_num -1
	incr b_num -1
	incr c_num -1
	
	set Xmults [linsert $Xmults 0 [expr 1.0/9]]

	set CurrTm 0
	set CurrCount 0
	if {$numta > $numtb} {
		set CurrTm [expr [lindex $tripa_tms [expr $numta - 1]] * [lindex $Gmults 0]]
		set CurrCount [lindex $Gmults 0]
  }
	
	for {set i 1} {$i < 7} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]		

	#Now set Ch A Y(N-2), B X(N-2), C G(N-2)
	incr a_num -1
	incr b_num -1
	incr c_num -1
	
	set Gmults [linsert $Gmults 0 0.0]

	set CurrTm 0
	set CurrCount 0
	if {$numta > $numtb} {
		set CurrTm [expr [lindex $tripa_tms [expr $numta - 1]] * [lindex $Ymults 0]]
		set CurrCount [lindex $Ymults 0]
  }
	
	for {set i 1} {$i < 7} {incr i} {
	  set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
    set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
		set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
    set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
		set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
	}
	
	set CurrTm [expr $CurrTm / $CurrCount]
	set cha_tms($a_num) $CurrTm
	set cha_codes($a_num) [retGXY "$a_num"]
	set chb_tms($b_num) $CurrTm
	set chb_codes($b_num) [retGXY "$b_num"]
	set chc_tms($c_num) $CurrTm
	set chc_codes($c_num) [retGXY "$c_num"]		

  #Finally, set Ch A X(N-2), B G(N-2), C Y(N-3) if appropriate:
	if {$numta == $numtb} {
	
		incr a_num -1
		incr b_num -1
		incr c_num -1

		set Ymults [linsert $Ymults 0 0.0]

		set CurrTm 0
		set CurrCount 0

		for {set i 1} {$i < 7} {incr i} {
	  	set CurrTm [expr $CurrTm + [lindex $tripa_tms [expr $numtb - $i]] * [lindex $Xmults $i]]
    	set CurrTm [expr $CurrTm + [lindex $tripb_tms [expr $numtb - $i]] * [lindex $Gmults $i]]
			set CurrTm [expr $CurrTm + [lindex $tripc_tms [expr $numtb - $i]] * [lindex $Ymults $i]]
    	set CurrCount [expr $CurrCount + [lindex $Gmults $i]]
			set CurrCount [expr $CurrCount + [lindex $Xmults $i]]
			set CurrCount [expr $CurrCount + [lindex $Ymults $i]]		
		}

		set CurrTm [expr $CurrTm / $CurrCount]
		set cha_tms($a_num) $CurrTm
		set cha_codes($a_num) [retGXY "$a_num"]
		set chb_tms($b_num) $CurrTm
		set chb_codes($b_num) [retGXY "$b_num"]
		set chc_tms($c_num) $CurrTm
		set chc_codes($c_num) [retGXY "$c_num"]		
  }
}

proc retthreelet { onelett } {
  switch $onelett {
	  A {set tl ALA}
		C {set tl CYS}
		D {set tl ASP}
		E {set tl GLU}
		F {set tl PHE}
		G {set tl GLY}
		H {set tl HIS}
		I {set tl ILE}
		K {set tl LYS}
		L {set tl LEU}
		M {set tl MET}
		N {set tl ASN}
    O {set tl HYP}
		P {set tl PRO}
		Q {set tl GLN}
		R {set tl ARG}
		S {set tl SER}
		T {set tl THR}
		V {set tl VAL}
		W {set tl TRP}
		Y {set tl TYR}
		default {set tl UNK}
	}
	return $tl
}


proc drawchain { canv } {

  global cha_tms chb_tms chc_tms
  global tripa_tms tripb_tms tripc_tms
	global ymult xmult locmax locmin canvheight chaintoshow
	global helcutoff_numaa helcutoff_tm
	global cha_heltype chb_heltype chc_heltype
	global chaina chainb chainc

	set numresa [array size cha_tms]
	set numresb [array size chb_tms]
	set numresc [array size chc_tms]

  if {$chaintoshow == {A}} {
	  set numres $numresa
		array set curr_tms [array get cha_tms]
		array set curr_heltype [array get cha_heltype]
		set curr_chain $chaina
	}  elseif {$chaintoshow == {B}} {
	  set numres $numresb
		array set curr_tms [array get chb_tms]
		array set curr_heltype [array get chb_heltype]
		set curr_chain $chainb
	} else {
  	set numres $numresc
		array set curr_tms [array get chc_tms]
		array set curr_heltype [array get chc_heltype]
		set curr_chain $chainc
	}

	for {set i 0} {$i < $numres} {incr i} {
    set currtm $curr_tms($i)
		set curry [expr ($locmax - $currtm) * $ymult]
		set currx [expr ($i * $xmult)]
    lappend xylist $currx $curry
		if {$xmult != 1} {
		  set currsize [expr floor($xmult/2)]
			set gxypos [expr $i % 3]
			if {$curr_heltype($i) == 0} {
			  $canv create rectangle [expr $currx-$currsize] [expr ($curry + 10)] [expr $currx+$currsize] [expr ($curry - 10)] -fill grey -outline grey -tag maindisp
			} elseif {$curr_heltype($i) == -0.5} {
			  $canv create rectangle [expr $currx-$currsize] [expr ($curry + 10)] [expr $currx+$currsize] [expr ($curry - 10)] -fill orange -outline orange -tag maindisp			
			} elseif {$curr_heltype($i) == 0.5} {
			  $canv create rectangle [expr $currx-$currsize] [expr ($curry + 10)] [expr $currx+$currsize] [expr ($curry - 10)] -fill pink -outline pink -tag maindisp			
			}
			if {$gxypos == 0} {
			  set currcol green
			} elseif {$gxypos == 1} {
			  set currcol blue
				set currtrip [expr (($i + 2) / 3) - 1]
				if {$i < $numresa} {
				  set ay [expr ($locmax - [lindex $tripa_tms $currtrip]) * $ymult]
					$canv create oval [expr $currx-4] [expr $ay-2] [expr $currx] [expr $ay+2] -fill black -tag extradisp
				}
				if {$i < $numresb} {
				  set by [expr ($locmax - [lindex $tripb_tms $currtrip]) * $ymult]
				  $canv create oval [expr $currx-2] [expr $by-2] [expr $currx+2] [expr $by+2] -fill black -tag extradisp
				}
				if {$i < $numresc} {
	  			set cy [expr ($locmax - [lindex $tripc_tms $currtrip]) * $ymult]		
					$canv create oval [expr $currx] [expr $cy-2] [expr $currx+4] [expr $cy+2] -fill black -tag extradisp
			  }
			} else {
			  set currcol red
			}
			
			set letty [expr ($curry + (20)) + $gxypos * 10]
			if {$letty > $canvheight} {
			  set letty [expr ($curry - 40) + $gxypos * 10]
			}
						
			$canv create text $currx $letty -anchor n -text [string index $curr_chain $i] -fill $currcol -tag maindisp
			$canv create oval [expr $currx-$currsize] [expr $curry-$currsize] [expr $currx+$currsize] [expr $curry+$currsize] -fill $currcol -tag maindisp
		}
	}
  $canv create line $xylist -tag maindisp
	set cutoffy [expr ($locmax - $helcutoff_tm) * $ymult]
	$canv create line 0 $cutoffy $currx $cutoffy -width 1 -fill blue -tag maindisp
}

proc setheltype { } {

  global cha_tms chb_tms chc_tms
	global helcutoff_numaa helcutoff_tm
	global cha_heltype chb_heltype chc_heltype
  global donehelset
	global smoothtrans
	
	set numresa [array size cha_tms]
	set numresb [array size chb_tms]
	set numresc [array size chc_tms]
  
	set starttype 0
	set currhelix 0

  for {set i 0} {$i < $numresa} {incr i} {
	  set currtm $cha_tms($i)
		set oldhelix $currhelix
		if {$currtm < $helcutoff_tm && $currtm > 0} {
		  set currhelix 0
		} else {
		  set currhelix 1
		}
		if {$currhelix != $oldhelix} {
			if {[expr ($i - $starttype)] > $helcutoff_numaa} {
			  set heltoset $oldhelix
			} else {
			  set heltoset $currhelix
			}
			for {set j $starttype} {$j < $i} {incr j} {
        set cha_heltype($j) $heltoset
			}
			set starttype $i
		}
	}
	if {[expr $numresa - $starttype] > $helcutoff_numaa} {
	  set heltoset $currhelix
	} else {
		set heltoset [expr abs($currhelix - 1)]
	}
	for {set j $starttype} {$j < $numresa} {incr j} {
    set cha_heltype($j) $heltoset
	}

	set starttype 0
	set currhelix 0
  for {set i 0} {$i < $numresb} {incr i} {
	  set currtm $chb_tms($i)
		set oldhelix $currhelix
		if {$currtm < $helcutoff_tm} {
		  set currhelix 0
		} else {
		  set currhelix 1
		}
		if {$currhelix != $oldhelix} {
			if {[expr $i - $starttype] > $helcutoff_numaa} {
			  set heltoset $oldhelix
			} else {
			  set heltoset $currhelix
			}
			for {set j $starttype} {$j < $i} {incr j} {
        set chb_heltype($j) $heltoset
			}
			set starttype $i
		}
	}
	if {[expr $numresb - $starttype] > $helcutoff_numaa} {
	  set heltoset $currhelix
	} else {
		set heltoset [expr abs($currhelix - 1)]
	}
	for {set j $starttype} {$j < $numresb} {incr j} {
    set chb_heltype($j) $heltoset
	}

  set starttype 0
	set currhelix 0
  for {set i 0} {$i < $numresc} {incr i} {
	  set currtm $chc_tms($i)
		set oldhelix $currhelix
		if {$currtm < $helcutoff_tm} {
		  set currhelix 0
		} else {
		  set currhelix 1
		}
		if {$currhelix != $oldhelix} {
			if {[expr $i - $starttype] > $helcutoff_numaa} {
			  set heltoset $oldhelix
			} else {
			  set heltoset $currhelix
			}
			for {set j $starttype} {$j < $i} {incr j} {
        set chc_heltype($j) $heltoset
			}
			set starttype $i
		}
	}
	if {[expr $numresc - $starttype] > $helcutoff_numaa} {
	  set heltoset $currhelix
	} else {
		set heltoset [expr abs($currhelix - 1)]
	}
	for {set j $starttype} {$j < $numresc} {incr j} {
    set chc_heltype($j) $heltoset
	}
  if {$smoothtrans == {Y}} {
	  for {set i 1} {$i < $numresa} {incr i} {
      set currval 0
		  if {($cha_heltype([expr $i-1]) == 0) && ($cha_heltype($i) == 1)} {
		    set currval 0.5
			} elseif {($cha_heltype([expr $i-1]) == 1) && ($cha_heltype($i) == 0)} {
	      set currval -0.5	  
			}
      if {$currval != 0} {
				set cha_heltype([expr $i - 1]) $currval
				set cha_heltype($i) $currval
				set diff_i_1 [expr abs($cha_tms([expr $i-1]) - $helcutoff_tm)]
				set diff_i [expr abs($cha_tms([expr $i]) - $helcutoff_tm)]
				if {$diff_i < $diff_i_1} {
					set cha_heltype([expr $i + 1]) $currval
				} else {
					set cha_heltype([expr $i - 2]) $currval
				}	
			}
		}
		  for {set i 1} {$i < $numresb} {incr i} {
      set currval 0
		  if {($chb_heltype([expr $i-1]) == 0) && ($chb_heltype($i) == 1)} {
		    set currval 0.5
			} elseif {($chb_heltype([expr $i-1]) == 1) && ($chb_heltype($i) == 0)} {
	      set currval -0.5	  
			}
      if {$currval != 0} {
				set chb_heltype([expr $i - 1]) $currval
				set chb_heltype($i) $currval
				set diff_i_1 [expr abs($chb_tms([expr $i-1]) - $helcutoff_tm)]
				set diff_i [expr abs($chb_tms([expr $i]) - $helcutoff_tm)]
				if {$diff_i < $diff_i_1} {
					set chb_heltype([expr $i + 1]) $currval
				} else {
					set chb_heltype([expr $i - 2]) $currval
				}	
			}
		}
	  for {set i 1} {$i < $numresc} {incr i} {
      set currval 0
		  if {($chc_heltype([expr $i-1]) == 0) && ($chc_heltype($i) == 1)} {
		    set currval 0.5
			} elseif {($chc_heltype([expr $i-1]) == 1) && ($chc_heltype($i) == 0)} {
	      set currval -0.5	  
			}
      if {$currval != 0} {
				set chc_heltype([expr $i - 1]) $currval
				set chc_heltype($i) $currval
				set diff_i_1 [expr abs($chc_tms([expr $i-1]) - $helcutoff_tm)]
				set diff_i [expr abs($chc_tms([expr $i]) - $helcutoff_tm)]
				if {$diff_i < $diff_i_1} {
					set chc_heltype([expr $i + 1]) $currval
				} else {
					set chc_heltype([expr $i - 2]) $currval
				}	
			}
		}

	}
	set donehelset {yes}
}

proc plottms { cw } {
  
	global cha_tms chb_tms chc_tms
	global mean_tm maxprop minprop helcutoff_numaa helcutoff_tm
	global ymult xmult locmax locmin canvheight
	global tripa_tms tripb_tms tripc_tms
	global cha_heltype chb_heltype chc_heltype 
	global chaintoshow
	global smoothtrans
  global chaina chainb chainc
	
	set smoothtrans {N}
	
	# These parameters can be modified in order to fit any screen:
	set winwidth 900
	set winheight 350
	
	set helcutoff_numaa 4
  set helcutoff_tm $mean_tm
	
	setheltype
	
	set locmax [expr ceil($maxprop)]
	#set locmin [expr floor($minprop)]
	set locmin 0
	set numresa [string length $chaina]
	set numresb [string length $chainb]
	set numresc [string length $chainc]
	
	set canvheight [expr $winheight - 125]
  toplevel $cw -height [expr $winheight] -width $winwidth
	focus $cw
	grab $cw
  wm title $cw "Helical propensity and helix settings"

	frame $cw.row1 -height [expr $canvheight+45] -width $winwidth
	frame $cw.row2 -height 100 -width $winwidth
	frame $cw.row2.a -width $winwidth
	frame $cw.row2.b -width $winwidth
	frame $cw.row2.c -width $winwidth
	frame $cw.row2.d -width $winwidth
	frame $cw.row2.e -width $winwidth
				
	canvas $cw.canvreg -height [expr $canvheight+45] -width $winwidth -bg white
  set xmult 6
	set xoff 50
	set ymult [expr $canvheight/($locmax - $locmin)]
	$cw.canvreg configure -scrollreg [list -[expr ceil($xmult/2)] 0 [expr ($numresa * $xmult)] $canvheight] -xscrollcommand "$cw.xsb set"
	canvas $cw.yscale  -height [expr $canvheight+45] -width $xoff -bg white
		
	scrollbar $cw.xsb -command "$cw.canvreg xview" -orient horiz
  set rtbar [expr $xoff]
	set xunit [expr $xoff / 10]
	$cw.yscale create line $rtbar 0 $rtbar $canvheight -width 3
	for {set i [expr int($locmax)]} {$i >= $locmin} {incr i -1} {
   	set curry [expr ($locmax - $i) * $ymult]
		set ltbar [expr $rtbar - $xunit]
		set curwid 1
		if {[expr $i % 5] == 0} {
		  set ltbar [expr $rtbar - ($xunit * 2)]
			set curwid 3
			$cw.yscale create text [expr $ltbar-2] $curry -anchor e -text "$i"
		}
		$cw.yscale create line $rtbar $curry $ltbar $curry -width $curwid
	}

  #x scale offset (top of scale markings):
	set topbar [expr $canvheight + (floor($xmult / 2) + 2)] 

  set chaintoshow A  
	drawchain $cw.canvreg

	for {set i 0} {$i < $numresa} {incr i} { 
  	if {[expr ($i % 3)] == 1} {
  		set currx [expr ($i * $xmult)]
			set botbar [expr $topbar + $xmult]
			set curwid 1
			set currtrip [expr ($i + 2) / 3]
			if {[expr ($i + 2) % 15] == 0} {
				set botbar [expr $topbar + 2 * $xmult]
				set curwid 3
  			$cw.canvreg create text $currx [expr $botbar + 4] -anchor n -text $currtrip
			}
			$cw.canvreg create line $currx $topbar $currx $botbar -width $curwid -fill black
	$cw.canvreg create line [expr -floor($xmult / 2)] $topbar [expr ($numresa - 1) * $xmult + floor($xmult / 2)] $topbar -width 3
    }
  }
	$cw.canvreg create text [expr $currx / 2] [expr $topbar + (2 * $xmult) + 15] -anchor n -text "Triplet number."
	$cw.yscale create text 2 [expr ($winheight - 125) / 2 ] -anchor w -text "Tm*"
	label $cw.labminhel_aa -text "Minimum # AA for helicity change: "
	entry $cw.entminhel_aa -textvariable helcutoff_numaa -width 3
 	label $cw.labminhel_tm -text "Cutoff Tm (mean: $mean_tm): "
	entry $cw.entminhel_tm -textvariable helcutoff_tm -width 5
  button $cw.outchainbut -text "Output current chain to text file" -command { 
    if {$chaintoshow == {A}} {
  		set numtoout [array size cha_tms]
      set fid [open ChainA_prop.txt w]		
			for {set i 0} {$i < $numtoout} {incr i} {
      	set currtrip [expr int(floor($i / 3))]
				set currtriptm [lindex $tripa_tms $currtrip]
				set curronelett [string index $chaina $i]
				if {[expr $i % 3] == 2 && $curronelett == {P}} { 
			  	set curronelett O
				} 
		  	puts $fid "[expr $i + 1] [retthreelet $curronelett] [expr $currtrip + 1] $currtriptm $cha_tms($i)"
			} 
		} elseif {$chaintoshow == {B}} {
  		set numtoout [array size chb_tms]
      set fid [open ChainB_prop.txt w]		
			for {set i 0} {$i < $numtoout} {incr i} {
      	set currtrip [expr int(floor($i / 3))]
				set currtriptm [lindex $tripb_tms $currtrip]
				set curronelett [string index $chainb $i]
				if {[expr $i % 3] == 2 && $curronelett == {P}} { 
			  	set curronelett O
				} 
		  	puts $fid "[expr $i + 1] [retthreelet $curronelett] [expr $currtrip + 1] $currtriptm $chb_tms($i)"
			} 
		} else {
  		set numtoout [array size chc_tms]
      set fid [open ChainC_prop.txt w]		
			for {set i 0} {$i < $numtoout} {incr i} {
      	set currtrip [expr int(floor($i / 3)) + 1]
				set currtriptm [lindex $tripc_tms $currtrip]
				set curronelett [string index $chainc $i]
				if {[expr $i % 3] == 2 && $curronelett == {P}} { 
			  	set curronelett O
				} 
		  	puts $fid "[expr $i + 1] [retthreelet $curronelett] [expr $currtrip + 1] $currtriptm $chc_tms($i)"
			}
		}
		close $fid		
	}

	button $cw.updatebut -text "Update & redraw" -command { 
	  setheltype
		.tmwin.canvreg delete maindisp extradisp
		drawchain .tmwin.canvreg
	}
  # To be added:
  button $cw.colourbut -text "Set colours/window size" -command { 
	   setheltype
		.tmwin.canvreg delete maindisp extradisp
		drawchain .tmwin.canvreg
	}

  button $cw.donebut -text "Done" -command { 
	  setheltype
    destroy .tmwin        
	}
  label $cw.labchainsel -text "Chain to show:"
	radiobutton $cw.chainsela -text A -value A -variable chaintoshow
  radiobutton $cw.chainselb -text B -value B -variable chaintoshow
	radiobutton $cw.chainselc -text C -value C -variable chaintoshow
  label $cw.labsmoothtrans -text "Smooth transitions over three residues?"
	radiobutton $cw.smoothtransy -text {Yes} -value Y -variable smoothtrans
  radiobutton $cw.smoothtransn -text {No} -value N -variable smoothtrans
  pack $cw.row1 $cw.row2 -side top
	pack $cw.xsb -side bottom -fill x -in $cw.row1
	pack $cw.yscale -side left -in $cw.row1
	pack $cw.canvreg -side left -after $cw.yscale -in $cw.row1
  pack $cw.row2.a $cw.row2.b $cw.row2.c $cw.row2.d $cw.row2.e -side top -in $cw.row2
	pack $cw.labminhel_aa $cw.entminhel_aa -side left -in $cw.row2.a
	pack $cw.labminhel_tm $cw.entminhel_tm  -side left -in $cw.row2.b
	pack $cw.labsmoothtrans $cw.smoothtransy $cw.smoothtransn -side left -in $cw.row2.c
	pack $cw.labchainsel $cw.chainsela $cw.chainselb $cw.chainselc $cw.outchainbut -side left -in $cw.row2.d
#	pack $cw.updatebut $cw.colourbut $cw.donebut -side left -in $cw.row2.e
  pack $cw.updatebut $cw.donebut -side left -in $cw.row2.e
  tkwait window $cw  	
}

proc readparlist {filename helindex } {
  
	# Angular offsets
  global CA_Ang_Gly_X CA_Ang_X_Y CA_Ang_Y_Gly
  global CO_Ang_Gly_toCA CO_Ang_X_toCA CO_Ang_Y_toCA OC_Ang_Gly_toCA OC_Ang_X_toCA OC_Ang_Y_toCA N_Ang_Gly_toCA N_Ang_X_toCA N_Ang_Y_toCA CB_Ang_X_toCA CB_Ang_Y_toCA

  # Radii of given type (angstroms)
	global CA_Gly_R CA_X_R CA_Y_R CO_Gly_R CO_X_R CO_Y_R OC_Gly_R OC_X_R OC_Y_R N_Gly_R N_X_R N_Y_R CB_X_R CB_Y_R
  # delta-X for given shift
	global CA_dX_Gly_X CA_dX_X_Y CA_dX_Y_Gly 
	global CO_dX_Gly_toCA CO_dX_X_toCA CO_dX_Y_toCA OC_dX_Gly_toCA OC_dX_X_toCA OC_dX_Y_toCA N_dX_Gly_toCA N_dX_X_toCA N_dX_Y_toCA CB_dX_X_toCA CB_dX_Y_toCA 
  # Chain to Chain offset
  global CA_dX_Ch CA_Ang_Ch
  
	set fileid [open $filename r]
	while {[gets $fileid currline] >=0} {
	  if {[string compare -length 1 {;} $currline] != 0 && [string length  $currline] > 3} {
		  scan $currline "%s %f" currvarname currval
			if {[string length $currvarname] > 4} {
				set nametouse $currvarname
				append nametouse ( $helindex )
				set $nametouse $currval
			}
			 
		}
	} 
	close $fileid
}

proc readsmoothparlist {filename} {
  
	# angular and helical length offsets
  global nonsmooth_Ang_GX nonsmooth_Ang_XY nonsmooth_Ang_YG
  global nonsmooth_dX_GX nonsmooth_dX_XY nonsmooth_dX_YG
  global smooth_Ang_GXY smooth_Ang_XYG smooth_Ang_YGX
  global smooth_dX_GXY smooth_dX_XYG smooth_dX_YGX
	
	# chi1 angles for terminal atom prediction
	global chi1_X chi1_Y
	
	set fileid [open $filename r]
	while {[gets $fileid currline] >=0} {
	  if {[string compare -length 1 {;} $currline] != 0 && [string length  $currline] > 3} {
		  scan $currline "%s %s %f" currvarname currindex currval
			if {[string length $currvarname] > 4} {
				set nametouse $currvarname
				append nametouse ( $currindex )
				set $nametouse $currval
			}
			 
		}
	} 
	close $fileid
}

proc makecylpolchain { curr_chainnum } {
  
	global atsel curr_heltype currchain
	# outputs CP coords in each global variable alongside
	# residue # and atom label plus helix type for CA atoms
	global cylpol_res cylpol_heltype cylpol_at cylpol_r cylpol_theta cylpol_x

	# Angular offsets
  global CA_Ang_Gly_X CA_Ang_X_Y CA_Ang_Y_Gly
  global CO_Ang_Gly_toCA CO_Ang_X_toCA CO_Ang_Y_toCA OC_Ang_Gly_toCA OC_Ang_X_toCA OC_Ang_Y_toCA N_Ang_Gly_toCA N_Ang_X_toCA N_Ang_Y_toCA CB_Ang_X_toCA CB_Ang_Y_toCA

  # Radii of given type (angstroms)
	global CA_Gly_R CA_X_R CA_Y_R CO_Gly_R CO_X_R CO_Y_R OC_Gly_R OC_X_R OC_Y_R N_Gly_R N_X_R N_Y_R CB_X_R CB_Y_R
  # delta-X for given shift
	global CA_dX_Gly_X CA_dX_X_Y CA_dX_Y_Gly 
	global CO_dX_Gly_toCA CO_dX_X_toCA CO_dX_Y_toCA OC_dX_Gly_toCA OC_dX_X_toCA OC_dX_Y_toCA N_dX_Gly_toCA N_dX_X_toCA N_dX_Y_toCA CB_dX_X_toCA CB_dX_Y_toCA 
  # Chain to Chain offset
  global CA_dX_Ch CA_Ang_Ch
  # helicity transition offsets
  global nonsmooth_Ang_GX nonsmooth_Ang_XY nonsmooth_Ang_YG
  global nonsmooth_dX_GX nonsmooth_dX_XY nonsmooth_dX_YG
	global smooth_Ang_GXY smooth_Ang_XYG smooth_Ang_YGX
  global smooth_dX_GXY smooth_dX_XYG smooth_dX_YGX
  global smoothtrans
	  	
	array unset cylpol_res 
	array unset cylpol_at
	array unset cylpol_r 
	array unset cylpol_theta 
	array unset cylpol_x
	
  set currmax [array size curr_heltype]
	set XOffSet [expr ($curr_chainnum - 1) * $CA_dX_Ch($curr_heltype(0))]
  set CurrAng [expr ($curr_chainnum - 1) * $CA_Ang_Ch($curr_heltype(0))]
  set at_in 0
	set smoothind 0
	set delta_Ang_Trip [expr (($CA_Ang_Gly_X(1) + $CA_Ang_X_Y(1) + $CA_Ang_Y_Gly(1)) - ($CA_Ang_Gly_X(0) + $CA_Ang_X_Y(0) + $CA_Ang_Y_Gly(0))) / 3]
 	set delta_dX_Trip [expr (($CA_dX_Gly_X(1) + $CA_dX_X_Y(1) + $CA_dX_Y_Gly(1)) - ($CA_dX_Gly_X(0) + $CA_dX_X_Y(0) + $CA_dX_Y_Gly(0))) / 3]
  if {$curr_chainnum == 1} {
	  set smooth_dX_Ch_Ch [expr ($CA_dX_Ch(0) - $CA_dX_Ch(1)) / 3]
		set smooth_Ang_Ch_Ch [expr ($CA_Ang_Ch(0) - $CA_Ang_Ch(1)) / 3]
	} elseif {$curr_chainnum == 2} {
	  set smooth_dX_Ch_Ch 0
		set smooth_Ang_Ch_Ch 0
	} else {
	  set smooth_dX_Ch_Ch [expr -($CA_dX_Ch(0) - $CA_dX_Ch(1)) / 3]
		set smooth_Ang_Ch_Ch [expr -($CA_Ang_Ch(0) - $CA_Ang_Ch(1)) / 3]	
	}
	
	set heltype $curr_heltype(0)
	if {$currmax > 0} {
  	for {set i 0} {$i < $currmax} {incr i 3} {  
      			
    	# GLY assignment
      set oldheltype $heltype
    	set heltype $curr_heltype($i)
			if {[expr abs($heltype)] != 0.5} {
			  set smoothind 0
				set 0_mult [expr abs($heltype - 1)]
				set 1_mult $heltype
				set smooth_Ang 0
				set smooth_dX 0
				if {$heltype != $oldheltype && $smoothtrans == {N}} {
  				set ov_mult [expr ($oldheltype - $heltype)]
					set XOffSet [expr $XOffSet +  $ov_mult * $nonsmooth_dX_YG($curr_chainnum)]
  				set CurrAng [expr $CurrAng + $ov_mult * $nonsmooth_Ang_YG($curr_chainnum)]
				}
				set ov_mult 0
			} else {
			  incr smoothind
				if {$smoothind == 4} {
				  set smoothind 1
				}
				if {$smoothind == 1} {
				  set smooth_Ang [expr $smooth_Ang_GXY($curr_chainnum) / 3]
					set smooth_dX [expr $smooth_dX_GXY($curr_chainnum) / 3]
				} 
				set 0_mult [expr abs(((abs($heltype - 0.5)) * 4) - $smoothind)]
				set 1_mult [expr abs((($heltype + 0.5) * 4) - $smoothind)]
				#set 0_mult [expr ceil($heltype)]
				#set 1_mult [expr abs(floor($heltype))]
				set ov_mult [expr $heltype * 2]
			}
			
			#offset chains as necessary via smoothing to smoothed CA-positon
      if {$i > 0} {
    		set XOffSet [expr $XOffSet + ($0_mult * $CA_dX_Y_Gly(0) + $1_mult * $CA_dX_Y_Gly(1)) / ($0_mult + $1_mult)]
				set XOffSet [expr $XOffSet + $ov_mult * ($smooth_dX_Ch_Ch + $delta_dX_Trip + $smooth_dX)]
    		set CurrAng [expr $CurrAng + ($0_mult * $CA_Ang_Y_Gly(0) + $1_mult * $CA_Ang_Y_Gly(1)) / ($0_mult + $1_mult)]
				set CurrAng [expr $CurrAng + $ov_mult * ($smooth_Ang_Ch_Ch + $delta_Ang_Trip + $smooth_Ang)]
      }
    	if {$atsel != {C}} {
        set cylpol_res($at_in) $i
				set cylpol_at($at_in) N
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $N_dX_Gly_toCA(0) + $1_mult * $N_dX_Gly_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $N_Gly_R(0) + $1_mult * $N_Gly_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $N_Ang_Gly_toCA(0) + $1_mult * $N_Ang_Gly_toCA(1)) / ($0_mult + $1_mult)]
  			incr at_in
			}

      set cylpol_res($at_in) $i
			set cylpol_heltype($at_in) $heltype
			set cylpol_at($at_in) CA
      set cylpol_x($at_in) $XOffSet
			set cylpol_r($at_in) [expr ($0_mult * $CA_Gly_R(0) + $1_mult * $CA_Gly_R(1)) / ($0_mult + $1_mult)]
			set cylpol_theta($at_in) $CurrAng
			incr at_in

    	if {$atsel != {C}} {
        set cylpol_res($at_in) $i
				set cylpol_at($at_in) C
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $CO_dX_Gly_toCA(0) + $1_mult * $CO_dX_Gly_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $CO_Gly_R(0) + $1_mult * $CO_Gly_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $CO_Ang_Gly_toCA(0) + $1_mult * $CO_Ang_Gly_toCA(1)) / ($0_mult + $1_mult)]
				incr at_in
	      set cylpol_res($at_in) $i
  			set cylpol_at($at_in) O
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $OC_dX_Gly_toCA(0) + $1_mult * $OC_dX_Gly_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $OC_Gly_R(0) + $1_mult * $OC_Gly_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $OC_Ang_Gly_toCA(0) + $1_mult * $OC_Ang_Gly_toCA(1)) / ($0_mult + $1_mult)]
				incr at_in
			}

    	# X assignment
    	# Arbitrarily - use value from Gly to set offset between Gly & X residues
      set oldheltype $heltype
    	set heltype $curr_heltype([expr $i +1])
			if {[expr abs($heltype)] != 0.5} {
			  set smoothind 0
				set 0_mult [expr abs($heltype - 1)]
				set 1_mult $heltype
				set smooth_Ang 0
				set smooth_dX 0
				if {$heltype != $oldheltype && $smoothtrans == {N}} {
  				set ov_mult [expr ($oldheltype - $heltype)]
					set XOffSet [expr $XOffSet + $ov_mult * $nonsmooth_dX_GX($curr_chainnum)]
  				set CurrAng [expr $CurrAng + $ov_mult * $nonsmooth_Ang_GX($curr_chainnum)]
				}
				set ov_mult 0

			} else {
			  incr smoothind
				if {$smoothind == 4} {
				  set smoothind 1
				}
				if {$smoothind == 1} {
				  set smooth_Ang [expr $smooth_Ang_XYG($curr_chainnum) / 3]
				  set smooth_dX [expr $smooth_dX_XYG($curr_chainnum) / 3]
				} 
				set 0_mult [expr abs(((abs($heltype - 0.5)) * 4) - $smoothind)]
				set 1_mult [expr abs((($heltype + 0.5) * 4) - $smoothind)]
				set ov_mult [expr $heltype * 2]
			}

    	set XOffSet [expr $XOffSet + ($0_mult * $CA_dX_Gly_X(0) + $1_mult * $CA_dX_Gly_X(1)) / ($0_mult + $1_mult)]
			set XOffSet [expr $XOffSet + $ov_mult * ($smooth_dX_Ch_Ch + $delta_dX_Trip + $smooth_dX)]
    	set CurrAng [expr $CurrAng + ($0_mult * $CA_Ang_Gly_X(0) + $1_mult * $CA_Ang_Gly_X(1)) / ($0_mult + $1_mult)]
			set CurrAng [expr $CurrAng + $ov_mult * ($smooth_Ang_Ch_Ch + $delta_Ang_Trip + $smooth_Ang)]

    	if {$atsel != {C}} {
        set cylpol_res($at_in) [expr $i + 1]
				set cylpol_at($at_in) N
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $N_dX_X_toCA(0) + $1_mult * $N_dX_X_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $N_X_R(0) + $1_mult * $N_X_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $N_Ang_X_toCA(0) + $1_mult * $N_Ang_X_toCA(1)) / ($0_mult + $1_mult)]
				incr at_in
			}

      set cylpol_res($at_in) [expr $i + 1]
			set cylpol_heltype($at_in) $heltype
			set cylpol_at($at_in) CA
      set cylpol_x($at_in) $XOffSet
			set cylpol_r($at_in) [expr ($0_mult * $CA_X_R(0) + $1_mult * $CA_X_R(1)) / ($0_mult + $1_mult)]
			set cylpol_theta($at_in) $CurrAng
			incr at_in


    	if {$atsel != {C}} {
        set cylpol_res($at_in) [expr $i + 1]
				set cylpol_at($at_in) C
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $CO_dX_X_toCA(0) + $1_mult * $CO_dX_X_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $CO_X_R(0) + $1_mult * $CO_X_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $CO_Ang_X_toCA(0) + $1_mult * $CO_Ang_X_toCA(1)) / ($0_mult + $1_mult)]
				incr at_in
	      set cylpol_res($at_in) [expr $i + 1]
  			set cylpol_at($at_in) O
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $OC_dX_X_toCA(0) + $1_mult * $OC_dX_X_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $OC_X_R(0) + $1_mult * $OC_X_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $OC_Ang_X_toCA(0) + $1_mult * $OC_Ang_X_toCA(1)) / ($0_mult + $1_mult)]
				incr at_in
			}
      
			if { $atsel == {A} } {
		    if {[string index $currchain [expr $i + 1]] != {G}} {
	      	set cylpol_res($at_in) [expr $i + 1]
  				set cylpol_at($at_in) CB
  				set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $CB_dX_X_toCA(0) + $1_mult * $CB_dX_X_toCA(1)) / ($0_mult + $1_mult)]
					set cylpol_r($at_in) [expr ($0_mult * $CB_X_R(0) + $1_mult * $CB_X_R(1)) / ($0_mult + $1_mult)]
 	  			set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $CB_Ang_X_toCA(0) + $1_mult * $CB_Ang_X_toCA(1)) / ($0_mult + $1_mult)]
					incr at_in
				}
			}

    	# Y assignment
      set oldheltype $heltype
    	set heltype $curr_heltype([expr $i + 2])
			if {[expr abs($heltype)] != 0.5} {
			  set smoothind 0
				set 0_mult [expr abs($heltype - 1)]
				set 1_mult $heltype
				set smooth_Ang 0
				set smooth_dX 0
				if {$heltype != $oldheltype && $smoothtrans == {N}} {
  				set ov_mult [expr ($oldheltype - $heltype)]
					set XOffSet [expr $XOffSet + $ov_mult * $nonsmooth_dX_XY($curr_chainnum)]
  				set CurrAng [expr $CurrAng + $ov_mult * $nonsmooth_Ang_XY($curr_chainnum)]
				}
				set ov_mult 0
			} else {
			  incr smoothind
				if {$smoothind == 4} {
				  set smoothind 1
				}
				if {$smoothind == 1} {
				  set smooth_Ang [expr $smooth_Ang_YGX($curr_chainnum) / 3]
				  set smooth_dX [expr $smooth_dX_YGX($curr_chainnum) / 3]
				} 
				set 0_mult [expr abs(((abs($heltype - 0.5)) * 4) - $smoothind)]
				set 1_mult [expr abs((($heltype + 0.5) * 4) - $smoothind)]
				set ov_mult [expr $heltype * 2]
			}

			# set up X & Angle for Y-position & offset chains as necessary 
			# via smoothing to smoothed CA-positon
    	set XOffSet [expr $XOffSet + (($0_mult * $CA_dX_X_Y(0) + $1_mult * $CA_dX_X_Y(1)) / ($0_mult + $1_mult))]
			set XOffSet [expr $XOffSet + $ov_mult * ($smooth_dX_Ch_Ch + $delta_dX_Trip + $smooth_dX)]
    	set CurrAng [expr $CurrAng + (($0_mult * $CA_Ang_X_Y(0) + $1_mult * $CA_Ang_X_Y(1)) / ($0_mult + $1_mult))]
			set CurrAng [expr $CurrAng + $ov_mult * ($smooth_Ang_Ch_Ch + $delta_Ang_Trip + $smooth_Ang)]

    	if {$atsel != {C}} {
        set cylpol_res($at_in) [expr $i + 2]
				set cylpol_at($at_in) N
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $N_dX_Y_toCA(0) + $1_mult * $N_dX_Y_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $N_Y_R(0) + $1_mult * $N_Y_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $N_Ang_Y_toCA(0) + $1_mult * $N_Ang_Y_toCA(1)) / ($0_mult + $1_mult)]
				incr at_in
			}

      set cylpol_res($at_in) [expr $i + 2]
			set cylpol_heltype($at_in) $heltype
			set cylpol_at($at_in) CA
      set cylpol_x($at_in) $XOffSet
			set cylpol_r($at_in) [expr ($0_mult * $CA_Y_R(0) + $1_mult * $CA_Y_R(1)) / ($0_mult + $1_mult)]
			set cylpol_theta($at_in) $CurrAng
			incr at_in

    	if {$atsel != {C}} {
        set cylpol_res($at_in) [expr $i + 2]
				set cylpol_at($at_in) C
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $CO_dX_Y_toCA(0) + $1_mult * $CO_dX_Y_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $CO_Y_R(0) + $1_mult * $CO_Y_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $CO_Ang_Y_toCA(0) + $1_mult * $CO_Ang_Y_toCA(1)) / ($0_mult + $1_mult)]
				incr at_in
	      set cylpol_res($at_in) [expr $i + 2]
  			set cylpol_at($at_in) O
  			set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $OC_dX_Y_toCA(0) + $1_mult * $OC_dX_Y_toCA(1)) / ($0_mult + $1_mult)]
				set cylpol_r($at_in) [expr ($0_mult * $OC_Y_R(0) + $1_mult * $OC_Y_R(1)) / ($0_mult + $1_mult)]
				set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $OC_Ang_Y_toCA(0) + $1_mult * $OC_Ang_Y_toCA(1)) / ($0_mult + $1_mult)]
				incr at_in
			}
      
			if { $atsel == {A} } {
			  if {[string index $currchain [expr $i + 2]] != {G}} {
	      	set cylpol_res($at_in) [expr $i + 2]
  				set cylpol_at($at_in) CB
  				set cylpol_x($at_in) [expr $XOffSet + ($0_mult * $CB_dX_Y_toCA(0) + $1_mult * $CB_dX_Y_toCA(1)) / ($0_mult + $1_mult)]
					set cylpol_r($at_in) [expr ($0_mult * $CB_Y_R(0) + $1_mult * $CB_Y_R(1)) / ($0_mult + $1_mult)]
					set cylpol_theta($at_in) [expr $CurrAng + ($0_mult * $CB_Ang_Y_toCA(0) + $1_mult * $CB_Ang_Y_toCA(1)) / ($0_mult + $1_mult)]
					incr at_in
				}
			}
		}
  }
}

proc outtopdb { fid At AtNum Res Chain ResNum xco yco zco } {
	 
	 set atid [string index $At 0]
   puts $fid [format {%-6s%5i  %-3s%4s%2s%4i    %8.3f%8.3f%8.3f%6.2f%6.2f%12s  } ATOM $AtNum $At $Res $Chain $ResNum $xco $yco $zco 1.0 0.0 $atid]
		 
}


proc outhelix { } {

	global chaina chainb chainc
	global cha_heltype chb_heltype chc_heltype 
  global cha_codes chb_codes chc_codes
	global chvar
	global coordtype outname resnum prohyp
  global cylpol_res cylpol_heltype cylpol_at cylpol_r cylpol_theta cylpol_x
	global outatnum
	global curr_heltype currchain
	global chi1_X chi1_Y
  
	set pi 3.141592654
	
	set fileid [open $outname w]
	set chi1name $outname 
	append chi1name ".X1"
	set chi1file [open $chi1name w]

  set outatnum 1
	for {set chain 1} {$chain < 4} {incr chain} {
	  set outresnum 1
		array unset curr_heltype
		array unset currcodes
		if {$chain == 1} {
      array set curr_heltype [array get cha_heltype]
			set currchain $chaina
		  array set currcodes [array get cha_codes]
			set chainlab A
		  if {$resnum == {B}} {
			  set outresnum $chvar(A_NT)
			}		
		} elseif {$chain == 2} {
      array set curr_heltype [array get chb_heltype]
			set currchain $chainb
		  array set currcodes [array get chb_codes]
			set chainlab B
		  if {$resnum == {B}} {
			  set outresnum $chvar(B_NT)
			}
		} else {
      array set curr_heltype [array get chc_heltype]
			set currchain $chainc
			array set currcodes [array get chc_codes]
			set chainlab C
		  if {$resnum == {B}} {
			  set outresnum $chvar(C_NT)
			}
		}
    makecylpolchain $chain
		set numats [array size cylpol_res]
		if {$numats > 0} {
      set oldres -1 
			incr outresnum -1
			for {set i 0} {$i < $numats} {incr i} {
        set currres $cylpol_res($i)
				if {$currres != $oldres} {
				  set curraa [string index $currchain $currres]
					set currcode $currcodes($currres)
					if {$curraa == {P} && $currcode == {Y} && $prohyp == {A}} {
					  set curraa O
					}
					set threelet [retthreelet $curraa]
					incr outresnum 1
					set oldres $currres
				}
				set currx $cylpol_x($i)
				set curry $cylpol_r($i)
				set currz $cylpol_theta($i) 
				set currz [expr $currz - (360 * floor($currz / 360))]
				set currz [expr $currz + (360 * ($currz < 0))]
				if {$coordtype == {A}} {
        	set cy1 [expr cos($pi * $currz / 180) * $curry]
					set currz [expr sin($pi * $currz / 180) * $curry]
					set curry $cy1
				}
				set currat $cylpol_at($i)
        outtopdb $fileid $currat $outatnum $threelet $chainlab $outresnum $currx $curry $currz
				if {$currat == {CA} && $currcode != {G}} {
  				set chi1var {chi1_}
	   			append chi1var $currcode
  				set chi1val [lindex [array get $chi1var $cylpol_heltype($i)] 1]
          puts $chi1file [format {%3s %1s %4i %5f} $threelet $chainlab $outresnum $chi1val]				
				}
				incr outatnum
			}
      puts $fileid [format {%-6s%5i  %7s%2s%-58i} TER $outatnum $threelet $chainlab $outresnum]	
			incr outatnum
		}
	}
  puts $fileid [format {%-80s} END]
  close $fileid    
	close $chi1file
}

proc genhelix { ghw } {

	global atsel coordtype outname resnum prohyp outfilename
		
  toplevel $ghw
	focus $ghw
	grab $ghw
  wm title $ghw "Generate triple-helical coordinates"
	frame $ghw.row1
	frame $ghw.row2
	frame $ghw.row3
	frame $ghw.row4
	frame $ghw.row5
	frame $ghw.row6
	
  label $ghw.atsellab -text "Atoms to produce:"
	radiobutton $ghw.atsela -text {N,CA,C,O,CB} -value A -variable atsel
  radiobutton $ghw.atselb -text {N,CA,C,O} -value B -variable atsel
	radiobutton $ghw.atselc -text {CA} -value C -variable atsel
	$ghw.atsela select
	label $ghw.coordtypelab -text "Coordinate type:"
	radiobutton $ghw.coordtypea -text Cartesian -value A -variable coordtype
  radiobutton $ghw.coordtypeb -text {Cylindrical polar} -value B -variable coordtype
	$ghw.coordtypea select
	label $ghw.resnumlab -text "Number residues starting at:"
	radiobutton $ghw.resnuma -text 1 -value A -variable resnum
  radiobutton $ghw.resnumb -text {Offset by sequence} -value B -variable resnum
	$ghw.resnuma select
	label $ghw.prohyplab -text "Turn Pro at Y position into Hyp:"
	radiobutton $ghw.prohypa -text Yes -value A -variable prohyp
  radiobutton $ghw.prohypb -text No -value B -variable prohyp
	$ghw.prohypa select
	label $ghw.outnamelab -text "Output file name: "
	button $ghw.fileforoutname -text "Select file" -command {
	  set outname [tk_getSaveFile -title "Output PDB file" -parent .ghwin -defaultextension pdb]
	}
	entry $ghw.outnameentry -width 40 -relief sunken -textvariable outname
	
  button $ghw.genhelbut -text "Produce PDB file" -command {
	  if {[string length $outname] > 0} {
		  outhelix
			set outfilename $outname
		} else {
		  tk_messageBox -type ok -message "You must specify a file name before proceeding." -parent .ghwin
		}
	}
	button $ghw.cancelbut -text "Done/Cancel" -command "destroy $ghw"

	pack $ghw.outnamelab $ghw.fileforoutname $ghw.outnameentry -in $ghw.row5 -side left
  pack $ghw.row1 $ghw.row2 $ghw.row3 $ghw.row4 $ghw.row5 $ghw.row6 -side top
	pack $ghw.atsellab $ghw.atsela $ghw.atselb $ghw.atselc -side left -in $ghw.row1
	pack $ghw.coordtypelab $ghw.coordtypea $ghw.coordtypeb -side left -in $ghw.row2
	pack $ghw.resnumlab $ghw.resnuma $ghw.resnumb -side left -in $ghw.row3
	pack $ghw.prohyplab $ghw.prohypa $ghw.prohypb -side left -in $ghw.row4
	pack $ghw.genhelbut $ghw.cancelbut -side left -in $ghw.row6
  tkwait window $ghw
	
}

# Procedures necessary for AT prediction

proc fillATlist { } {

  global AAList ATList MaxAT
		
	set currcount -1
	set fileid [open {ATlist.dat} r]
	while {[gets $fileid currline] >=0} {
	  if {[string compare -length 1 {;} $currline] != 0 && [string length  $currline] > 3} {
		  scan $currline "%s %s" curraa currat
			if {[string length $curraa] == 3} {
        incr currcount
			  set AAList($currcount) $curraa
				set ATList($currcount) $currat
			}
		}
	} 
	close $fileid
	set MaxAT $currcount
}

proc fillATregions { } {

  global AAList ATList MaxAT
	global PropList dRList zAbList zInList
	global x1gm x1gp x1t
	
	for {set i 0} {$i <= $MaxAT} {incr i} {
	  set fileid [open "ATdata/$AAList($i)$ATList($i)SSAcc.txt" r]
		set numregs 0
		if {$fileid > 0} {
			while {[gets $fileid currline] >=0} {
	    	if {[string compare -length 1 {;} $currline] != 0 && [string length  $currline] > 3} {
		    	if {$numregs == 0} {
  					set numcounts [scan $currline "%s %i %i %i %i %i %i %i" currfield r(0) r(1) r(2) r(3) r(4) r(5) r(6)]
	  				if {$currfield == {Number}} {
		  		  	set numregs [expr $numcounts - 1]
							set totcount 0
							for {set j 0} {$j < $numregs} {incr j} {
						  	incr totcount $r($j)
							}
							for {set j 0} {$j < $numregs} {incr j} {
						  	lappend "PropList($i)" [expr double($r($j)) / $totcount]
							}
						}
						set donefloats 0
					} else {
            if {$donefloats == 0} {
              if {$currline == {X1GM has 0}} {
						  	for {set j 1} {$j <= $numregs} {incr j} {
						  		lappend "x1gm($i)" 0
								}
							} elseif {$currline == {X1GP has 0}} {
						  	for {set j 1} {$j <= $numregs} {incr j} {
						  		lappend "x1gp($i)" 0
								}
							} elseif {$currline == {X1T has 0}} {
						  	for {set j 1} {$j <= $numregs} {incr j} {
						  		lappend "x1t($i)" 0
								}
								set donefloats 1
							}
					  	scan $currline "%s %f %f %f %f %f %f %f %f" currfield r(0) r(1) r(2) r(3) r(4) r(5) r(6) r(7)
							if {$currfield == {Mode(dR)}} {
						  	for {set j 0} {$j < $numregs} {incr j} {
						  		lappend "dRList($i)" $r($j)
								}
							} elseif {$currfield == {Mode(zAb)}} {
						  	for {set j 0} {$j < $numregs} {incr j} {
						  		lappend "zAbList($i)" $r($j)
								}
							} elseif {$currfield == {Mode(zIn)}} {
						  	for {set j 0} {$j < $numregs} {incr j} {
						  		lappend "zInList($i)" $r($j)
								}
							} elseif {$currfield == {%X1GM}} {
						  	for {set j 1} {$j <= $numregs} {incr j} {
						  		lappend "x1gm($i)" $r($j)
								}
							} elseif {$currfield == {%X1GP}} {
						  	for {set j 1} {$j <= $numregs} {incr j} {
						  		lappend "x1gp($i)" $r($j)
								}
						  }	elseif {$currfield == {%X1T}} {
						  	for {set j 1} {$j <= $numregs} {incr j} {
						  		lappend "x1t($i)" $r($j)
								}
								set donefloats 1	
							}
						}
					}
				}
			}
		}
    close $fileid
  } 
}

# specify pdb file name/path and the chain desired.
# if file only has a single chain, or all chains are wanted, specify 0 for chain 
proc readpdb { fname chainspec } {

  global ResList ResNumList ChainList NCoList CACoList CCoList OCoList CBCoList GXYList maxListind
	
	set currcount -1
	set oldResNum -1
	set prevchain 1
	set GXYstr {GXY}
	set GXYind -1
	set fileid [open $fname r]
	while {[gets $fileid currline] >=0} {
	  if {[string compare -length 4 {ATOM} $currline] == 0} {
		  scan $currline "%*6s%5i%*c%4s%c%3s%*c%c%4i%c%*c%*c%*c%8f%8f%8f%6f%6f" AtNum At AltLoc Res Chain ResNum AChar xco yco zco Occ BFac
      set Chain [format %c $Chain]
      if {$chainspec == {0} || $Chain == $chainspec} {
			  if {$oldResNum != $ResNum} {
				  incr currcount
					set oldResNum $ResNum
					set ResList($currcount) $Res
					set ResNumList($currcount) $ResNum
				  set ChainList($currcount) $Chain
					if {$prevchain != $Chain} {
					  set GXYind -1
					}
					incr GXYind
					set prevchain $Chain
					set GXYList($currcount) [string index $GXYstr [expr $GXYind % 3]]
				}
				if {$At == {N}} {
			    set NCoList($currcount) [list $xco $yco $zco]
				} elseif {$At == {CA}} {
			    set CACoList($currcount) [list $xco $yco $zco]
				} elseif {$At == {C}} {
			    set CCoList($currcount) [list $xco $yco $zco]
				} elseif {$At == {O}} {
			    set OCoList($currcount) [list $xco $yco $zco]
				} elseif {$At == {CB}} {
			    set CBCoList($currcount) [list $xco $yco $zco]
				}
			}
			
		}
	} 
	close $fileid
	set maxListind $currcount
	
}

proc readchi1 { fname } {

  global chi1List
	
	set fileid [open $fname r]
	while {[gets $fileid currline] >=0} {
    scan $currline "%3s %1s %4i %5f" threelet chainlab resnum chi1val
		set chi1List($chainlab,$resnum) $chi1val
	} 
	close $fileid
}

proc capAAs { } {

  global AAList MaxAT
  
	for {set i 0} {$i <= $MaxAT} {incr i} {
	  set AAList($i) [string toupper $AAList($i)]
	}
	
}

# returns an angle in radians given one in degrees
proc degrad { Ang } {

  return [expr ($Ang / 180.0) * 3.14159265]
	
}

# returns an angle in degrees given one in radians 
proc raddeg { Ang } {

  return [expr ($Ang * 180.0) / 3.14159265]
	
}

# Vector addition of A + B 
proc vecadd { A B } {
  
	set numdim [llength $A]
	for {set i 0} {$i < $numdim} {incr i} {
	  lappend retvec [expr [lindex $A $i] + [lindex $B $i]]
	}
  return $retvec
}

# Vector subtraction of A - B 
proc vecsubt { A B } {
  
	set numdim [llength $A]
	for {set i 0} {$i < $numdim} {incr i} {
	  lappend retvec [expr [lindex $A $i] - [lindex $B $i]]
	}
  return $retvec
}

# Vector multiplication of A by scalar S 
proc vecmult { A S } {
  
	set numdim [llength $A]
	for {set i 0} {$i < $numdim} {incr i} {
	  lappend retvec [expr [lindex $A $i] * $S]
	}
  return $retvec
}

# Vector division of A by scalar S 
proc vecdiv { A S } {
  
	set numdim [llength $A]
	for {set i 0} {$i < $numdim} {incr i} {
	  lappend retvec [expr [lindex $A $i] / $S]
	}
  return $retvec
}

# Vector norm of A 
proc norm { A } {
  
	set numdim [llength $A]
	set sumsq 0.0
	for {set i 0} {$i < $numdim} {incr i} {
	  set sumsq [expr $sumsq + pow([lindex $A $i], 2)]
	}
	return [expr sqrt($sumsq)]
}

# Vector cross product - A x B - A and B must be 3 dimensional vectors.
proc crossp { A B } {
  
	lappend retvec [expr [lindex $A 1] * [lindex $B 2] - [lindex $A 2] * [lindex $B 1]]
	lappend retvec [expr [lindex $A 2] * [lindex $B 0] - [lindex $A 0] * [lindex $B 2]]
	lappend retvec [expr [lindex $A 0] * [lindex $B 1] - [lindex $A 1] * [lindex $B 0]]
  return $retvec

}

# Vector dot product of A.B 
proc dotp { A B } {
  
	set numdim [llength $A]
	set dotprod 0.0
	for {set i 0} {$i < $numdim} {incr i} {
	  set dotprod [expr $dotprod + [lindex $A $i] * [lindex $B $i]]
	}
  return $dotprod
}

proc quadeqn { a b c } {

  set rootterm [expr pow($b, 2) - 4 * $a * $c]
	if { $rootterm < 0 } {
	  return 0
	}
	set root [expr sqrt($rootterm)]
	lappend retvec [expr (-$b + $root) / (2 * $a)]
	lappend retvec [expr (-$b - $root) / (2 * $a)]
	
	return $retvec
}

proc altinplunitvecs { V1 V2 Ang } {

  set NVo [crossp $V2 $V1]
  set NV  [vecdiv $NVo [norm $NVo]]

  set DP [expr [norm $V1] * cos([degrad $Ang])]

  set A [lindex $NV 0]
  set B [lindex $NV 1]
  set C [lindex $NV 2]
  set ax [lindex $V1 0]
	set ay [lindex $V1 1]
	set az [lindex $V1 2]
  set AB2 [expr pow($A, 2) + pow($B, 2)]
  set BC2 [expr pow($B, 2) + pow($C, 2)]
  set CyBz [expr $C * $ay - $B * $az]
  set BxAy [expr $B * $ax - $A * $ay]

  set Coef2 [expr $AB2 * pow($CyBz, 2) + $BC2 * pow($BxAy, 2) + 2 * $A * $C * $CyBz * $BxAy]
  set Coef1 [expr 2 * $B * $DP * ($A * $C * $BxAy + $AB2 * $CyBz)]
  set Coef0 [expr pow($B, 2) * ($AB2 * pow($DP, 2) - pow($BxAy,2))]

  #if {[expr abs($Coef2) > 0.001]} {
    
		set bzs [quadeqn $Coef2 $Coef1 $Coef0]
		if {[llength $bzs] != 2} {
		  
			puts stdout {Complex root - altinplunitvecs}
      return [list 0] 		
		} else {

		  set bz1 [lindex $bzs 0]
			set bz2 [lindex $bzs 1]
      set bx1 [expr ($B * $DP + $CyBz * $bz1) / $BxAy]
      set bx2 [expr ($B * $DP + $CyBz * $bz2) / $BxAy]
      set by1 [expr -($A * $bx1 + $C * $bz1) / $A]
      set by2 [expr -($A * $bx2 + $C * $bz2) / $A]

		}
		
#	} else {
#	  puts stdout {Bad root in altinplvecs.}
#		return [list 0]
#	}

  return [list $bx1 $by1 $bz1 $bx2 $by2 $bz2]
}

proc inplunitvecs { V1 V2 Ang } {

  set NVo [crossp $V2 $V1]
  set NV  [vecdiv $NVo [norm $NVo]]
  set DP [expr [norm $V1] * cos([degrad $Ang])]
  	
  set A [lindex $NV 0]
  set B [lindex $NV 1]
  set C [lindex $NV 2]
  set ax [lindex $V1 0]
	set ay [lindex $V1 1]
	set az [lindex $V1 2]
  set AB2 [expr pow($A, 2) + pow($B, 2)]
  set AC2 [expr pow($A, 2) + pow($C, 2)]
  set CxAz [expr $C * $ax - $A * $az]
  set AyBx [expr $A * $ay - $B * $ax]

  set Coef2 [expr $AB2 * pow($CxAz, 2) + $AC2 * pow($AyBx, 2) + 2 * $B * $C * $AyBx * $CxAz]
  set Coef1 [expr 2 * $A * $DP * ($B * $C * $AyBx + $AB2 * $CxAz)]
  set Coef0 [expr pow($A, 2) * ($AB2 * pow($DP, 2) - pow($AyBx,2))]
  
#  if {[expr abs($Coef2) > 0.001]} {
    
		set bzs [quadeqn $Coef2 $Coef1 $Coef0]
		if {[llength $bzs] != 2} {
		  
#			puts stdout {Complex root. Going to altinplunitvecs}
			return [altinplunitvecs $V1 $V2 $Ang]
		
		} else {

		  set bz1 [lindex $bzs 0]
			set bz2 [lindex $bzs 1]
      set by1 [expr ($A * $DP + $CxAz * $bz1) / $AyBx]
      set by2 [expr ($A * $DP + $CxAz * $bz2) / $AyBx]
      set bx1 [expr -($B * $by1 + $C * $bz1) / $A]
      set bx2 [expr -($B * $by2 + $C * $bz2) / $A]	  
		}
		
	#} else {
	#  puts stdout {Bad root. Going to altinplvecs.}
	#	return [altinplunitvecs $V1 $V2 $Ang]
	#}

  return [list $bx1 $by1 $bz1 $bx2 $by2 $bz2]
}

proc fixAT { NCo CACo CCo dR zAb zIn } {

	set normNCo [vecsubt $NCo $CACo]
  set normCACo [list 0.0 0.0 0.0]
	set normCCo [vecsubt $NCo $CCo]
	set At_Ang [inplunitvecs $normNCo $normCCo $zIn]
	if {[llength $At_Ang] == 6} {
		set PlEq [crossp $normNCo $normCCo]
  	set nToAA [crossp [lrange $At_Ang 0 2] $normNCo]
  	set AASpec 1
  	if {[expr [dotp $PlEq $nToAA] * $zIn] < 0} {
	  	set AASpec 0
		}
  	set iUnitVec [lrange $At_Ang [expr $AASpec * 3] [expr $AASpec * 3 + 2]]
  	set At_Ang [inplunitvecs $iUnitVec $PlEq $zAb]
		if {[llength $At_Ang] != 6} {
		  puts stdout {Couldn't set rUnitVec.}
			return [list 0 0 0]
		}
  	set AASpec 1
  	if {[expr [dotp $PlEq [lrange $At_Ang 0 2]] * $zAb] < 0} {
	  	set AASpec 0
		}
  	set rUnitVec [lrange $At_Ang [expr $AASpec * 3] [expr $AASpec * 3 + 2]]
  	set normRCo [vecmult $rUnitVec $dR]
  	return [vecadd $normRCo $CACo]
	}
  puts stdout {Couldn't set iUnitVec.}
	return [list 0 0 0]
}

proc modProps { } {
  global PropList MaxAT MaxPropAT x1gm x1gp x1t
	
	for {set i 0} {$i <= $MaxAT} {incr i} {
	  set currelms $PropList($i)
		set currtot [llength $currelms]
		set runtot 0.0
    set maxelm 0
		set currmax 0
		if {$currtot > 1} {
			for {set  j 0} {$j < [expr $currtot - 1]} {incr j} {
		  	set currprop [lindex $currelms $j]
				set runtot [expr $runtot + $currprop]
				lappend newelms $runtot
				if {$currprop > $currmax} {
				  set maxelm $j
					set currmax $currprop
				}
			}
		}
		if {[lindex $currelms [expr $currtot - 1]] > $currmax} {
		  set maxelm [expr $currtot - 1]
		}
		lappend newelms 1
		set PropList($i) $newelms
		set MaxPropAT($i) $maxelm
		unset newelms
		
	  set currelms $x1gm($i)
		set currtot [llength $currelms]
    set runtot 0.0
		set ovtot 0.0
		set zeroflag 1
		if {$currtot > 1} {
			for {set  j 0} {$j < $currtot} {incr j} {
		  	set currprop [lindex $currelms $j]
				set ovtot [expr $ovtot + $currprop]
				if {$currprop != 0} {
					set zeroflag 0
				}
			}
			for {set  j 0} {$j < [expr $currtot - 1]} {incr j} {
		  	if {$zeroflag == 0} {
					set currprop [expr [lindex $currelms $j] / $ovtot]
					set runtot [expr $runtot + $currprop]
					lappend newelms $runtot
				}
			}
		}
    if {$zeroflag == 0} {
  		lappend newelms 1.0
			set x1gm($i) $newelms
		}
		unset -nocomplain newelms

	  set currelms $x1gp($i)
		set currtot [llength $currelms]
    set runtot 0.0
		set ovtot 0.0
		set zeroflag 1
		if {$currtot > 1} {
			for {set  j 0} {$j < $currtot} {incr j} {
		  	set currprop [lindex $currelms $j]
				set ovtot [expr $ovtot + $currprop]
				if {$currprop != 0} {
					set zeroflag 0
				}
			}
			for {set  j 0} {$j < [expr $currtot - 1]} {incr j} {
		  	if {$zeroflag == 0} {
					set currprop [expr [lindex $currelms $j] / $ovtot]
					set runtot [expr $runtot + $currprop]
					lappend newelms $runtot
				}
			}
		}
    if {$zeroflag == 0} {
  		lappend newelms 1.0
			set x1gp($i) $newelms
		}
		unset -nocomplain newelms

	  set currelms $x1t($i)
		set currtot [llength $currelms]
    set runtot 0.0
		set ovtot 0.0
		set zeroflag 1
		if {$currtot > 1} {
			for {set  j 0} {$j < $currtot} {incr j} {
		  	set currprop [lindex $currelms $j]
				set ovtot [expr $ovtot + $currprop]
				if {$currprop != 0} {
					set zeroflag 0
				}
			}
			for {set  j 0} {$j < [expr $currtot - 1]} {incr j} {
		  	if {$zeroflag == 0} {
					set currprop [expr [lindex $currelms $j] / $ovtot]
					set runtot [expr $runtot + $currprop]
					lappend newelms $runtot
				}
			}
		}
    if {$zeroflag == 0} {
  		lappend newelms 1.0
			set x1t($i) $newelms
		}
		unset -nocomplain newelms
	}
}

proc getProp { currlist } {
  
	set randnum [expr rand()]
  set testin 0 
	while {$randnum >= [lindex $currlist $testin]} {
		incr testin
	}
	return $testin
}

proc threedangle { Vector1 Vector2 } {

  set ACosTest [expr [dotp $Vector1 $Vector2] / ([norm $Vector1] * [norm $Vector2])]

	if {$ACosTest > 1} {
	  if {[expr abs($ACosTest) - 1] < 0.05} {
		  set ACosTest [expr round($ACosTest)]
		}
  }

  set ang [expr acos($ACosTest)]

  return [raddeg $ang]

}

proc sendtoout {file at atnum res chain resnum cartco} {

  global coordtypeAT

  set xco [lindex $cartco 0]
	if {$coordtypeAT == {B}} {
    set yco [expr sqrt(pow([lindex $cartco 1], 2) + pow([lindex $cartco 2], 2))]
		set zco [raddeg [expr atan2([lindex $cartco 2], [lindex $cartco 1])]]
		if {$zco < 0} {
		  set zco [expr $zco + 360]
		}
	} else {
	  set yco [lindex $cartco 1]
		set zco [lindex $cartco 2]
	}
	outtopdb $file $at $atnum $res $chain $resnum $xco $yco $zco

}

# Set ProbMeth as code given in paper:
# MP - most probable

proc writeATs { rootfile ProbMeth numwrite fnumoffset } {

  global AAList ATList
	global PropList dRList zAbList zInList MaxPropAT
  global ResList ResNumList ChainList NCoList CACoList CCoList OCoList maxListind chi1List
	global bball
  
	set bbfilename $rootfile
	append bbfilename "_bb.pdb"
	set bbfile [open $bbfilename w]
	for {set i 1} {$i <= $numwrite} {incr i} {
	  set fnum [expr $i + $fnumoffset]
		set atfilename $rootfile
		append atfilename "_AT_"
		append atfilename "$fnum"
		append atfilename ".pdb"
		set ATfile($i) [open $atfilename w]
	}
	set tempAAinds [array get AAList]
  set maxAAinds [llength $tempAAinds]
	for {set i 0} {$i < [expr $maxAAinds / 2]} {incr i} {
	  lappend AAinds [lindex $tempAAinds [expr [lsearch $tempAAinds $i] + 1]]
	}
	set atnum 0
  for {set i 0} {$i <= $maxListind} {incr i} {
    puts stdout "Writing residue number [expr $i + 1] of [expr $maxListind + 1]"
    set currres $ResList($i)
		set currchain $ChainList($i)
		set currresnum $ResNumList($i)
		incr atnum
		sendtoout $bbfile N $atnum $currres $currchain $currresnum $NCoList($i)
    incr atnum
    sendtoout $bbfile {CA} $atnum $currres $currchain $currresnum $CACoList($i)
		incr atnum
		sendtoout $bbfile C $atnum $currres $currchain $currresnum $CCoList($i)
    incr atnum
		sendtoout $bbfile O $atnum $currres $currchain $currresnum $OCoList($i)
    if {$bball == {Y}} {
			for {set k 0} {$k < $numwrite} {incr k} {
				incr atnum -3
				sendtoout $ATfile([expr $k + 1]) N $atnum $currres $currchain $currresnum $NCoList($i)
    		incr atnum
				sendtoout $ATfile([expr $k + 1]) {CA} $atnum $currres $currchain $currresnum $CACoList($i)
				incr atnum
				sendtoout $ATfile([expr $k + 1]) C $atnum $currres $currchain $currresnum $CCoList($i)
    		incr atnum
				sendtoout $ATfile([expr $k + 1]) O $atnum $currres $currchain $currresnum $OCoList($i)
				set currATelms [lsearch -all $AAinds $ResList($i)]
				set numATs [llength $currATelms]
			}
    }
		set currATelms [lsearch -all $AAinds $ResList($i)]
		set numATs [llength $currATelms]
		if {$numATs > 0} {
			for {set j 0} {$j < $numATs} {incr j} {
		  	set currind [lindex $currATelms $j]
				unset -nocomplain indlist
				if {$ProbMeth == {MP} || $currres == {ALA}} {
				  set currprop $MaxPropAT($currind)
				}
				
				for {set k 0} {$k < $numwrite} {incr k} {
				  if {$ProbMeth == {PC} && $currres != {ALA}} {
					  set currprop [getProp $PropList($currind)]
					} elseif {$ProbMeth == {CHI1} && $currres != {ALA}} {
            set x1list [list 0]
						set currx1 $chi1List($currchain,$currresnum)
            #using +/- 45 degree range for each canonical chi1-rotamer angle
						if {$currx1 > -105 && $currx1 < -15} {
						  set x1list x1gm($currind)
						} elseif {$currx1 > 15 && $currx1 < 105} {
						  set x1list x1gp($currind)
						} elseif {[expr abs($currx1)] > 135 && [expr abs($currx1)] <= 180 } {
						  set x1list x1t($currind)
						}
            set x1elms [llength $x1list]
						if {$currx1 == 500 || [lindex $x1list [expr $x1elms - 1]] == 0} {
						  set x1list $PropList($currind)
						}
						set currprop [getProp $x1list]
					}
					lappend indlist $currprop
				}
				incr atnum
				for {set k 0} {$k < $numwrite} {incr k} {
          set indlistind [lindex $indlist $k]
					set currloc [fixAT $NCoList($i) $CACoList($i) $CCoList($i)  [lindex $dRList($currind) $indlistind] [lindex $zAbList($currind) $indlistind] [lindex $zInList($currind) $indlistind]]
					sendtoout $ATfile([expr $k + 1]) $ATList($currind) $atnum $currres $currchain $currresnum $currloc
				}
			}
		}		
	}
  close $bbfile
	for {set i 1} {$i <= $numwrite} {incr i} {
	  close $ATfile($i)
	}
}

proc addATs { atw } {

	global outfilename coordtype outbase numtooutput probmeth coordtypeAT bball
  global curroutblock numoutblocks		
  toplevel $atw
	focus $atw
	grab $atw
  wm title $atw "Predict side-chain terminal atoms"
	frame $atw.row1
	frame $atw.row2
	frame $atw.row3
	frame $atw.row4
	frame $atw.row5
	frame $atw.row6
 	frame $atw.row7
	
	set numtooutput 100	
  label $atw.description -text "Produce an ensemble of AT predictions with: "
	label $atw.outnamelab -text "Base output file name : "
	button $atw.fileforoutname -text "Select file" -command {
	  set outbase [tk_getSaveFile -title "Output PDB file" -parent .atwin -defaultextension pdb]
	}
	entry $atw.outnameentry -width 40 -relief sunken -textvariable outbase
	
	label $atw.numfileslab -text "Number of output files : "
	entry $atw.numfilesentry -width 5 -relief sunken -textvariable numtooutput
	label $atw.probmethlab -text "Probabilistic choice:"
	radiobutton $atw.probmethX1 -text {Chi1 rotamer} -value {CHI1} -variable probmeth
  radiobutton $atw.probmethPC -text {Proportion of all counted ATs} -value {PC} -variable probmeth
  radiobutton $atw.probmethML -text {Most likely} -value {MP} -variable probmeth
	$atw.probmethX1 select
	label $atw.coordtypelab -text "Coordinate type:"
	radiobutton $atw.coordtypea -text Cartesian -value A -variable coordtypeAT
  radiobutton $atw.coordtypeb -text {Cylindrical polar} -value B -variable coordtypeAT
	$atw.coordtypea select
	label $atw.bblab -text "Include backbone in all files:"
	radiobutton $atw.bby -text Yes -value Y -variable bball
  radiobutton $atw.bbn -text No -value N -variable bball
	$atw.bbn select
  button $atw.genatbut -text "Produce ensemble of files" -command {
	  if {[string length $outname] > 0} {
			if {$probmeth == {MP} && $numtooutput != 1} {
  		  tk_messageBox -type ok -message "Outputting a single AT file since most likely specified." -parent .atwin
				set numtooutput 1
			}
      set numoutblocks [expr int(floor($numtooutput / 200))]
			for {set curroutblock 0} {$curroutblock < $numoutblocks} {incr curroutblock} {
			  writeATs "$outbase" $probmeth 200 [expr $curroutblock * 200]
			}
			set curroutblock [expr $numtooutput - ($numoutblocks * 200)]
 		  writeATs "$outbase" $probmeth $curroutblock [expr $numoutblocks * 200]
		} else {
		  tk_messageBox -type ok -message "You must specify a file name before proceeding." -parent .atwin
		}
	}
	button $atw.cancelbut -text "Done/Cancel" -command "destroy $atw"

  pack $atw.row1 $atw.row2 $atw.row3 $atw.row4 $atw.row5 $atw.row6 $atw.row7 -side top
	pack $atw.description -side left -in $atw.row1
	pack $atw.outnamelab $atw.fileforoutname $atw.outnameentry -in $atw.row2 -side left
	pack $atw.numfileslab $atw.numfilesentry -side left -in $atw.row3
	pack $atw.probmethlab $atw.probmethX1 $atw.probmethPC $atw.probmethML -side left -in $atw.row4
	pack $atw.coordtypelab $atw.coordtypea $atw.coordtypeb -side left -in $atw.row5
	pack $atw.bblab $atw.bby $atw.bbn -side left -in $atw.row6
	pack $atw.genatbut $atw.cancelbut -side left -in $atw.row7
  tkwait window $atw
	
}

proc retonelet { threelett } {
  switch $threelett {
	  {ALA} {set ol A}
		{CYS} {set ol C}
		{ASP} {set ol D}
		{GLU} {set ol E}
		{PHE} {set ol F}
		{GLY} {set ol G}
		{HIS} {set ol H}
		{ILE} {set ol I}
		{LYS} {set ol K}
		{LEU} {set ol L}
		{MET} {set ol M}
		{ASN} {set ol N}
    {HYP} {set ol O}
		{PRO} {set ol P}
		{GLN} {set ol Q}
		{ARG} {set ol R}
		{SER} {set ol S}
		{THR} {set ol T}
		{VAL} {set ol V}
		{TRP} {set ol W}
		{TYR} {set ol Y}
		default {set ol X}
	}
	return $ol
}

proc writeHYPs { rootfile scwfilename scwfrfilename } {

  global AAList ATList
	global dRList zAbList zInList maxListind
  global ResList ResNumList ChainList NCoList CACoList CCoList OCoList CBCoList GXYList
  global predPro predHyp predAla
	global bball
  
	set bbfilename $rootfile
	set bbfile [open $bbfilename w]
  if {"$scwfilename" != ""} {
  	set scwfile [open $scwfilename w]
	}
  if {"$scwfrfilename" != ""} {
  	set scwfrfile [open $scwfrfilename w]
	}

	set tempAAinds [array get AAList]
  set maxAAinds [llength $tempAAinds]
	for {set i 0} {$i < [expr $maxAAinds / 2]} {incr i} {
	  lappend AAinds [lindex $tempAAinds [expr [lsearch $tempAAinds $i] + 1]]
	}
	set atnum 0
	set HyfATelms [lsearch -all $AAinds {HYF}]
	set numHyfATs [llength $HyfATelms]
	set PrfATelms [lsearch -all $AAinds {PRF}]
	set numPrfATs [llength $PrfATelms]
      
  for {set i 0} {$i <= $maxListind} {incr i} {
    puts stdout "Writing residue number [expr $i + 1] of [expr $maxListind + 1]"
    set currres $ResList($i)
		set currchain $ChainList($i)
		set currresnum $ResNumList($i)
    if {$bball == {Y}} {
			incr atnum
			sendtoout $bbfile N $atnum $currres $currchain $currresnum $NCoList($i)
    	incr atnum
    	sendtoout $bbfile {CA} $atnum $currres $currchain $currresnum $CACoList($i)
			incr atnum
			sendtoout $bbfile C $atnum $currres $currchain $currresnum $CCoList($i)
    	incr atnum
			sendtoout $bbfile O $atnum $currres $currchain $currresnum $OCoList($i)
    }
		if {$currres == {HYP} && $predHyp == {Y}} {
      if {"$scwfilename" != ""} {
     	  puts -nonewline $scwfile o
 	    }
			for {set j 0} {$j < $numHyfATs} {incr j} {
		  	set currind [lindex $HyfATelms $j]
				set currprop 0
				incr atnum
#        set indlistind [lindex $indlist $k]
				set currloc [fixAT $NCoList($i) $CACoList($i) $CCoList($i)  [lindex $dRList($currind) $currprop] [lindex $zAbList($currind) $currprop] [lindex $zInList($currind) $currprop]]
				sendtoout $bbfile $ATList($currind) $atnum $currres $currchain $currresnum $currloc
					if {"$scwfrfilename" != ""} {
						sendtoout $scwfrfile $ATList($currind) $atnum $currres $currchain $currresnum $currloc
					}
			}
		} elseif {$currres == {PRO} && $predPro == {Y} && $GXYList($i) == {X}} {
      if {"$scwfilename" != ""} {
     	  puts -nonewline $scwfile p
 	    }
			for {set j 0} {$j < $numPrfATs} {incr j} {
		  	set currind [lindex $PrfATelms $j]
				set currprop 0
				incr atnum
#        set indlistind [lindex $indlist $k]
				set currloc [fixAT $NCoList($i) $CACoList($i) $CCoList($i)  [lindex $dRList($currind) $currprop] [lindex $zAbList($currind) $currprop] [lindex $zInList($currind) $currprop]]
				sendtoout $bbfile $ATList($currind) $atnum $currres $currchain $currresnum $currloc
			}
		} elseif {$currres == {ALA} && $predAla == {Y}} {
      if {"$scwfilename" != ""} {
     	  puts -nonewline $scwfile a
 	    }
			incr atnum
			sendtoout $bbfile CB $atnum $currres $currchain $currresnum $CBCoList($i)
		} else {
      if {"$scwfilename" != ""} {
    	  puts -nonewline $scwfile [retonelet $currres]
	    }
		}		
	}
  close $bbfile
	if {"$scwfilename" != ""} {
    close $scwfile
	}
	if {"$scwfrfilename" != ""} {
    close $scwfrfile
	}

}

proc addHYP { ahw } {

	global outfilename coordtype outbase predPro predHyp predAla coordtypeAT bball doSCW SCWfile
  global curroutblock numoutblocks
  toplevel $ahw
	focus $ahw
	grab $ahw
  wm title $ahw "Predict full side-chains"
	frame $ahw.row1
	frame $ahw.row2
	frame $ahw.row3
	frame $ahw.row4
	frame $ahw.row5
	frame $ahw.row6
	frame $ahw.row7
	frame $ahw.row8
	frame $ahw.row9
	frame $ahw.row10
	frame $ahw.row11
	frame $ahw.row12
				
	label $ahw.outnamelab -text "Output PDB file name : "
	button $ahw.fileforoutname -text "Select file" -command {
	  set outbase [tk_getSaveFile -title "Output PDB file" -parent .ahwin -defaultextension pdb]
	}
	entry $ahw.outnameentry -width 40 -relief sunken -textvariable outbase

	label $ahw.scwlab -text "SCWRL files:"
	radiobutton $ahw.scwy -text Yes -value Y -variable SCWfile
  radiobutton $ahw.scwn -text No -value N -variable SCWfile
  $ahw.scwn select
	
	label $ahw.scwnamelab -text "SCWRL sequence file name : "
	button $ahw.fileforscwname -text "Select file" -command {
	  set scwname [tk_getSaveFile -title "SCWRL sequence file" -parent .ahwin -defaultextension seq]
	}
	entry $ahw.scwnameentry -width 40 -relief sunken -textvariable scwname

	label $ahw.scwfrnamelab -text "SCWRL frame file name : "
	button $ahw.fileforscwfrname -text "Select file" -command {
	  set scwfrname [tk_getSaveFile -title "SCWRL frame file" -parent .ahwin -defaultextension frame]
	}
	entry $ahw.scwfrnameentry -width 40 -relief sunken -textvariable scwfrname
	
	label $ahw.predHyplab -text "Hyp in Y:"
	radiobutton $ahw.predHypY -text Yes -value Y -variable predHyp
	radiobutton $ahw.predHypN -text No -value N -variable predHyp
	$ahw.predHypY select
	label $ahw.predProlab -text "Pro in X:"
	radiobutton $ahw.predProY -text Yes -value Y -variable predPro
	radiobutton $ahw.predProN -text No -value N -variable predPro
	$ahw.predProY select
	label $ahw.predAlalab -text "Ala in X/Y:"
	radiobutton $ahw.predAlaY -text Yes -value Y -variable predAla
	radiobutton $ahw.predAlaN -text No -value N -variable predAla
	$ahw.predAlaY select


	label $ahw.coordtypelab -text "Coordinate type:"
	radiobutton $ahw.coordtypea -text Cartesian -value A -variable coordtypeAT
  radiobutton $ahw.coordtypeb -text {Cylindrical polar} -value B -variable coordtypeAT
	$ahw.coordtypea select
	label $ahw.bblab -text "Include backbone in file:"
	radiobutton $ahw.bby -text Yes -value Y -variable bball
  radiobutton $ahw.bbn -text No -value N -variable bball
	$ahw.bby select
  button $ahw.genhyfbut -text "Produce file(s)" -command {
	  if {[string length $outname] > 0 && ([string length $scwname] > 0 || $SCWfile == {N})} {
      if {$SCWfile == {Y}} {
			  writeHYPs "$outbase" "$scwname" "$scwfrname"
		  } else {
			  writeHYPs "$outbase" "" ""
			}
		} else {
		  tk_messageBox -type ok -message "You must specify file name(s) before proceeding." -parent .ahwin
		}
	}
	button $ahw.cancelbut -text "Done/Cancel" -command "destroy $ahw"

  pack $ahw.row1 $ahw.row2 $ahw.row3 $ahw.row4 $ahw.row5 $ahw.row6 $ahw.row7 $ahw.row8 $ahw.row9 $ahw.row10 $ahw.row11 $ahw.row12 -side top
	pack $ahw.outnamelab $ahw.fileforoutname $ahw.outnameentry -in $ahw.row1 -side left
	pack $ahw.predHyplab $ahw.predHypY $ahw.predHypN -side left -in $ahw.row4
	pack $ahw.predProlab $ahw.predProY $ahw.predProN -side left -in $ahw.row5
	pack $ahw.predAlalab $ahw.predAlaY $ahw.predAlaN -side left -in $ahw.row6
	
	pack $ahw.coordtypelab $ahw.coordtypea $ahw.coordtypeb -side left -in $ahw.row7
	pack $ahw.bblab $ahw.bby $ahw.bbn -side left -in $ahw.row8
	pack $ahw.scwlab $ahw.scwy $ahw.scwn -side left -in $ahw.row9
	pack $ahw.scwnamelab $ahw.fileforscwname $ahw.scwnameentry -in $ahw.row10 -side left
	pack $ahw.scwfrnamelab $ahw.fileforscwfrname $ahw.scwfrnameentry -in $ahw.row11 -side left
	pack $ahw.genhyfbut $ahw.cancelbut -side left -in $ahw.row12
  tkwait window $ahw
	
}


# Main program

	set starta 0
	set enda 0
	set startb 0
	set endb 0
	set startc 0
	set endc 0
  	
	setupchvar
	global ntvar ctvar
  global chaina_raw chainb_raw chainc_raw
	global cha_file chb_file chc_file
  global chaina chainb chainc
  global donehelset outfilename coordtype atsel outfile

  # variables to clear for new files	
	global tripa_tms tripb_tms tripc_tms
	global mean_tm
	global cha_tms chb_tms chc_tms
  global cha_codes chb_codes chc_codes

	set donehelset {no}
	set builthelix {no}
	set outfilename {NA}
	  	
	frame .col1
	frame .col2
	frame .row1
	frame .row2
	frame .row3

  set copya 0
	set copyb 0
	set cha_file {}
  	
	label .chalab -text "Chain A (FASTA format): "
	entry .chaentry -width 30 -relief sunken -textvariable cha_file
	button .filefora -text "Select file" -command {

		if {$builthelix == {yes}} {
	    set resetfornew [tk_messageBox -default yes -type yesno -icon question -message "It looks like you have already built a helix. Do you want to reset & rerun to build another?" -parent . ]     
			if {$resetfornew == {yes}} {
			  destroy .
				exec {THeBuScr.1.07.tcl}
			}
		}
	  set cha_file [tk_getOpenFile -title "Chain A FASTA file" -parent .]
		unset -nocomplain copya
		set copya [tk_dialog .copya {Chain A set} {Use same sequence for chain(s):} question 0 None B "B & C"]
		if {$copya > 0} {
			unset -nocomplain chb_file
		  set chb_file $cha_file
			if {$copya == 2} {
				unset -nocomplain chc_file
			  set chc_file $cha_file
			}
		}
	}

	button .configcha -text "Select triple-helix" -command {
		set chaina_raw [openseq "$cha_file"]
		findgxx "$chaina_raw" starta enda
		dispseq .chainawin $chaina_raw $starta $enda A_NT A_CT
		if {$copya > 0} {
			set copyseq [tk_messageBox -type yesno -icon question -message "Use same N- and C-termini for sequence(s) using same file? (Note: this does not verify that files are still same)" -parent . ]
		  if {$copyseq == {yes}} {
        set chainb_raw $chaina_raw
				set chvar(B_NT) $chvar(A_NT)
				set chvar(B_CT) $chvar(A_CT)
				if {$copya == 2} {
				  set chainc_raw $chaina_raw
					set chvar(C_NT) $chvar(A_NT)
				  set chvar(C_CT) $chvar(A_CT)  
				}
			}
		}
	}

	label .chblab -text "Chain B (FASTA format): "
	entry .chbentry -width 30 -relief sunken -textvariable chb_file
	button .fileforb -text "Select file" -command {
	  set chb_file [tk_getOpenFile -title "Chain B FASTA file" -parent . ] 
		set copyb [tk_dialog .copyb {Chain B set} {Use same sequence for chain C} question 0 No Yes]
		if {$copyb == 1} {
		  set chc_file $chb_file
		}
	}
	button .configchb -text "Select triple-helix" -command {
		set chainb_raw [openseq "$chb_file"]
		findgxx "$chainb_raw" startb endb
		dispseq .chainbwin $chainb_raw $startb $endb B_NT B_CT
    if {$copyb > 0} {
			set copyseq [tk_messageBox -type yesno -icon question -message "Use same N- and C-termini for chain C? (Note: this does not verify C file is still same as B)" -parent . ]
		  if {$copyseq == {yes}} {
        set chainc_raw $chainb_raw
				set chvar(C_NT) $chvar(B_NT)
				set chvar(C_CT) $chvar(B_CT)
			}
		}
	}

	label .chclab -text "Chain C (FASTA format): "
	entry .chcentry -width 30 -relief sunken -textvariable chc_file
	button .fileforc -text "Select file" -command {
	  set chc_file [tk_getOpenFile -title "Chain C FASTA file" -parent . ] 
	}
	button .configchc -text "Select triple-helix" -command {
		set chainc_raw [openseq "$chc_file"]
		findgxx "$chainc_raw" startc endc
		dispseq .chaincwin $chainc_raw $startc $endc C_NT C_CT
	}

	button .doprops -text "Set/show helical params" -command {
  	set nsetflag 0
		foreach i [array names chvar] {
		  if {$chvar($i) == -1} {
			  set nsetflag 1
			}
    }
		if {$nsetflag == 0} {
			
      set lena [expr $chvar(A_CT) - $chvar(A_NT)]
			set lenb [expr $chvar(B_CT) - $chvar(B_NT)]
			set lenc [expr $chvar(C_CT) - $chvar(C_NT)]
			
			if { [expr abs($lena - $lenb)] > 3 || [expr abs($lenc - $lenb)] > 3 || [expr abs($lena - $lenc)] > 3 } {		    
				tk_messageBox -type ok -icon warning -message "A chain is set to be longer than another by more than one GXY triplet. Please re-select N- and C-terminal residues.Current settings: \n\nChain A N: $chvar(A_NT) C: $chvar(A_CT)\nChain B N: $chvar(B_NT) C: $chvar(B_CT)\nChain C N: $chvar(C_NT) C: $chvar(C_CT)" -parent .
			} else {

				set useterms [tk_messageBox -type okcancel -default ok -icon question -message "Current N- and C-terminal settings: \n\nChain A N: $chvar(A_NT) C: $chvar(A_CT)\nChain B N: $chvar(B_NT) C: $chvar(B_CT)\nChain C N: $chvar(C_NT) C: $chvar(C_CT)\n\n Proceed with these termini?" -parent . ]

				if {$useterms == {ok}} {
					#truncate chains

					set chaina "[string range $chaina_raw [expr ($chvar(A_NT)-1)] [expr ($chvar(A_CT)-1)]]"
    			set chainb "[string range $chainb_raw [expr ($chvar(B_NT)-1)] [expr ($chvar(B_CT)-1)]]"
					set chainc "[string range $chainc_raw [expr ($chvar(C_NT)-1)] [expr ($chvar(C_CT)-1)]]"
					orderchains
    			readpropensities
					unset -nocomplain tripa_tms
					unset -nocomplain tripb_tms
					unset -nocomplain tripc_tms
					unset -nocomplain mean_tm
	  			trippropensities $chaina $chainb $chainc
					unset -nocomplain cha_tms
					unset -nocomplain cha_codes
					unset -nocomplain chb_tms
					unset -nocomplain chb_codes
					unset -nocomplain chc_tms
					unset -nocomplain chc_codes
					boxcarpropensities $chaina $chainb $chainc
					plottms .tmwin
      	}	
			}
		}	else {
		  tk_messageBox -type ok -icon warning -message "N- and C-termini for each chain must be set first. Current settings: \n\nChain A N: $chvar(A_NT) C: $chvar(A_CT)\nChain B N: $chvar(B_NT) C: $chvar(B_CT)\nChain C N: $chvar(C_NT) C: $chvar(C_CT)" -parent .
    }
	}
	button .buildbut -text "Build helix" -command {
    if {$donehelset == {yes}} {
		  readparlist THparams_AR.dat 0
  	  readparlist THparams_IR.dat 1
			readsmoothparlist THparams_nonsmooth.dat
			readsmoothparlist THparams_smooth.dat
			readsmoothparlist THparams_chi1.dat
      genhelix .ghwin
			set builthelix {yes}
    } else {
			tk_messageBox -type ok -icon warning -message "Need to set helical parameters first." -parent .
		}
  }
	button .addsidebut -text "Add side-chain ATs" -command {
    if {$outfilename != {NA}} {
      fillATlist
			fillATregions
      if {$coordtype == {A} && $atsel != {C}} {
		  	readpdb "$outfilename" 0
			  readchi1 "$outfilename.X1"
			} else { 
   		  set coordtype A
				set atsel B
				set outname "temp.pdb"
				outhelix
		  	readpdb "temp.pdb" 0
			  readchi1 "temp.pdb.X1"
				set builthelix {yes}				
			}
			capAAs
			modProps
      addATs .atwin
    } else {
			tk_messageBox -type ok -icon warning -message "Need to set helical parameters & build helix first." -parent .
		}
  }

	button .addhypbut -text "Add all-atom PRO/HYP " -command {
    if {$outfilename != {NA}} {
      fillATlist
			fillATregions
      if {$coordtype == {A} && $atsel == {A}} {
		  	readpdb "$outfilename" 0
			  readchi1 "$outfilename.X1"
			} else { 
   		  set coordtype A
				set atsel A
				set outname "temp.pdb"
				outhelix
		  	readpdb "temp.pdb" 0
			  readchi1 "temp.pdb.X1"			
				set builthelix {yes}
			}
			capAAs
			modProps
      addHYP .ahwin
    } else {
			tk_messageBox -type ok -icon warning -message "Need to set helical parameters & build helix first." -parent .
		}
  }

	button .aboutbut -text "About" -command {
    set getlic [tk_dialog .abouteh {About} "THe BuScr version 1.07.\n\nAn interactive triple-helical collagen building script.\n\nCopyright 2003-2013, Jan K. Rainey.\n\nContact:\njan.rainey@dal.ca\n\nWebsite:\nstructbio.biochem.dal.ca/jrainey" {} 0 Return]	  
	}
	button .exitbut -text "Exit" -command {
    set reallyquit [tk_messageBox -default yes -type yesno -icon question -message "Really exit?" -parent . ]     
		if {$reallyquit == {yes}} {
		  destroy .
		}
	}
	button .resetbut -text "Restart & build new triple-helix" -command {
    set resetquest [tk_messageBox -default yes -type yesno -icon question -message "Really reset and start again?" -parent . ]     
		if {$resetquest == {yes}} {
		  destroy .
			exec {THeBuScr.1.07.tcl}
		}
	}

  wm title . {THe BuScr version 1.07}
	pack .col1 .col2 -side left
	pack .row1 .row2 .row3 .resetbut -in .col1 -side top
	pack .chalab .filefora .chaentry .configcha -in .row1 -side left
	pack .aboutbut .doprops .buildbut .addsidebut .addhypbut .exitbut -in .col2 -side top
	pack .chblab .fileforb .chbentry .configchb -in .row2 -side left
	pack .chclab .fileforc .chcentry .configchc -in .row3 -side left
	
